from collections import namedtuple
from openpyxl.utils.datetime import MAC_EPOCH, from_ISO8601
from openpyxl.utils import coordinate_to_tuple
from openpyxl.xml.functions import iterparse
from openpyxl.worksheet._reader import (
    WorkSheetParser,
    _cast_number,
    VALUE_TAG, FORMULA_TAG,
    COL_TAG, ROW_TAG, CF_TAG,
    FILTER_TAG, VALIDATION_TAG,
    PROPERTIES_TAG, VIEWS_TAG,
    FORMAT_TAG, HYPERLINK_TAG,
    MERGE_TAG,
    PROT_TAG, EXT_TAG,
    LEGACY_TAG, PRINT_TAG,
    MARGINS_TAG, PAGE_TAG,
    HEADER_TAG, ROW_BREAK_TAG,
    SCENARIOS_TAG, TABLE_TAG,
    INLINE_STRING,
)
from openpyxl.worksheet.filters import AutoFilter
from openpyxl.worksheet.hyperlink import HyperlinkList
from openpyxl.worksheet.merge import MergeCells
from openpyxl.worksheet.views import SheetViewList
from openpyxl.worksheet.datavalidation import DataValidationList
from openpyxl.worksheet.properties import WorksheetProperties
from openpyxl.worksheet.dimensions import (
    SheetFormatProperties,
)
from openpyxl.cell.text import Text
from .richtext import parse_richtext

CellRecord = namedtuple('CellRecord', ['row', 'column', 'value', 'style_id'])


class SynoWorkSheetParser(WorkSheetParser):

    def __init__(self, src, shared_strings, workbook):
        WorkSheetParser.__init__(self, src, shared_strings,
                                 data_only=workbook.data_only,
                                 epoch=workbook.epoch,
                                 date_formats=workbook._date_formats)
        self.workbook = workbook
        self.is_mac_calendar = self.epoch == MAC_EPOCH

    def parse(self):
        dispatcher = {
            COL_TAG: self.parse_column_dimensions,
            CF_TAG: self.parse_formatting,
            EXT_TAG: self.parse_extensions,
        }

        properties = {
            FILTER_TAG: ('auto_filter', AutoFilter),
            VALIDATION_TAG: ('data_validations', DataValidationList),
            PROPERTIES_TAG: ('sheet_properties', WorksheetProperties),
            VIEWS_TAG: ('views', SheetViewList),
            FORMAT_TAG: ('sheet_format', SheetFormatProperties),
            HYPERLINK_TAG: ('hyperlinks', HyperlinkList),
            MERGE_TAG: ('merged_cells', MergeCells),
        }

        unused_tags = set([
            PROT_TAG, LEGACY_TAG, PRINT_TAG,
            MARGINS_TAG, PAGE_TAG, HEADER_TAG, ROW_BREAK_TAG,
            SCENARIOS_TAG, TABLE_TAG,
        ])

        for _, element in iterparse(self.source):
            tag_name = element.tag
            if tag_name == ROW_TAG:
                row = self.parse_row(element)
                element.clear()
                yield row
            elif tag_name in dispatcher:
                dispatcher[tag_name](element)
                element.clear()
            elif tag_name in properties:
                prop = properties[tag_name]
                obj = prop[1].from_tree(element)
                setattr(self, prop[0], obj)
                element.clear()
            elif tag_name in unused_tags:
                element.clear()

    def parse_cell(self, element):
        data_type = element.get('t', 'n')
        coordinate = element.get('r')
        style_id = element.get('s', 0)
        if style_id:
            style_id = int(style_id)

        if data_type == "inlineStr":
            value = None
        else:
            value = element.findtext(VALUE_TAG, None) or None

        if coordinate:
            row, column = coordinate_to_tuple(coordinate)
        else:
            return None

        if element.find(FORMULA_TAG) is not None:
            value = self.parse_formula(element)
        elif value is not None:
            if data_type == 'n':
                value = _cast_number(value)
                if self.is_mac_calendar and style_id in self.date_formats and isinstance(
                        value, (int, float)):
                    value += 1492
            elif data_type == 's':
                value = self.shared_strings[int(value)]
            elif data_type == 'b':
                value = bool(int(value))
            elif data_type == "str":
                # formula string
                pass
            elif data_type == 'd':
                value = from_ISO8601(value)
        elif data_type == 'inlineStr':
            child = element.find(INLINE_STRING)
            if child is not None:
                value = parse_richtext(Text.from_tree(child), self.workbook)

        return CellRecord(row - 1, column - 1, value, style_id)


    def parse_extensions(self, element):
        # Office#2687
        uri_map = {
            '{CCE6A557-97BC-4B89-ADB6-D9C93CAAB3DF}': ('data_validations', DataValidationList)
        }
        for child in element:
            uri = child.get('uri').upper()
            if uri in uri_map and child:
                key, cls = uri_map[uri]
                if getattr(cls, key, None) is None:
                    setattr(self, key, cls.from_tree(child[0]))
