<?php
require_once '/var/packages/Spreadsheet/target/phpoffice/PHPExcel/IOFactory.php';
require_once '/var/packages/Spreadsheet/target/lib/const.php';
require_once '/var/packages/Spreadsheet/target/lib/utils.php';

function AddSheetPrefix($SheetIndex)
{
	return 'sh_' . $SheetIndex;
}

function RemoveSheetPrefix($SheetId)
{
	return substr($SheetId, 3);
}

function parseFormulaField($value) {
	if (is_numeric($value)) {
		return floatval($value);
	} else {
		return $value;
	}
}

// some chart series formula may contains parentheses
function RemoveParentheses($value) {
	$value = trim($value);
	while ($value[0] === '(' && $value[strlen($value) - 1] === ')') {
		$value = trim(substr($value, 1, -1));
	}
	return $value;
}

class PHPExcel_Writer_Snapshot {

	private $_jsSheet;

	public function __construct() {
		PHPExcel_Worksheet::setAllowInvalidTitle(true);
		PHPExcel_Worksheet::setDisableXfInheritance(true);
		PHPExcel_Worksheet_ColumnDimension::setGlobalMode(true);
	}

	private function saveIndex($folder, &$worksheets) {
		$total = count($worksheets);
		$jsIndex = array(
			'maxIndex' => $total,
			'order' => array(),
			'sheets' => array(),
			'schemaVersion' => SCHEMA_VERSION,
			'ver' => 1,
			'gcVer' => 1,
		);

		$count = 1;
		foreach ($worksheets as $worksheet) {
			$sheetId = AddSheetPrefix($count);
			$jsIndex['order'][] = $sheetId;
			$jsIndex['sheets'][$sheetId] = array(
				'title' => is_array($worksheet) ? $worksheet['worksheetName'] : $worksheet,
				'deleted' => false,
			);
			$count++;
		}
		if (! file_put_contents ( $folder . '/index', json_encode ( $jsIndex ) )) {
			OnError ( "Failed to save index file", __FILE__, __LINE__ );
		}
	}

	private function fillDefinedNames($folder, $phpExcel) {
		$definedNames = array();
		foreach ($phpExcel->getNamedRanges() as $namedRange) {
			$index = 'global';
			if ($namedRange->getLocalOnly()) {
				$index = AddSheetPrefix($namedRange->getLocalSheetId() + 1);
			}
			if (!isset($definedNames[$index])) {
				$definedNames[$index] = array();
			}
			$definedNames[$index][$namedRange->getName()] = $namedRange->getRawValue();
		}
		if (!file_put_contents($folder.'/define', json_encode($definedNames, JSON_FORCE_OBJECT))) {
			OnError ( "Failed to save defined names file", __FILE__, __LINE__ );
		}
	}

	private function fillStyles($folder, &$phpExcel) {
		$stylesheet = array(
			'fonts' => [],
			'fills' => [],
			'alignments' => [],
			'borders' => [],
			'numberFormats' => [],
			'styles' => [],
		);

		$fontHashTable = new PHPExcel_HashTable();
		$fillHashTable = new PHPExcel_HashTable();
		$alignmentHashTable = new PHPExcel_HashTable();
		$borderHashTable = new PHPExcel_HashTable();
		$numFmtHashTable = new PHPExcel_HashTable();

		// default alignemnt can be non empty
		$defaultStyle = $phpExcel->getDefaultStyle();
		$emptyAlignment = new PHPExcel_Style_Alignment();

		$defaultFontHashCode = $defaultStyle->getFont()->getHashCode();
		$defaultFillHashCode = $defaultStyle->getFill()->getHashCode();
		$defaultAlignmentHashCode = $emptyAlignment->getHashCode();
		$defaultBordersHashCode = $defaultStyle->getBorders()->getHashCode();
		$defaultNumberFormatHashCode = $defaultStyle->getNumberFormat()->getHashCode();
		$defaultQuotePrefix = $defaultStyle->getQuotePrefix();

		if ($defaultAlignmentHashCode !== $defaultStyle->getAlignment()->getHashCode()) {
			$alignmentHashTable->add($emptyAlignment);
		}

		foreach ($phpExcel->getCellXfCollection() as $cellXf) {
			$fontHashTable->add($cellXf->getFont());
			$fillHashTable->add($cellXf->getFill());
			$alignmentHashTable->add($cellXf->getAlignment());
			$borderHashTable->add($cellXf->getBorders());
			$numFmtHashTable->add($cellXf->getNumberFormat());
		}

		for ($i = 1, $count = count($phpExcel->getCellXfCollection()); $i < $count; ++$i) {
			$cellXf = $phpExcel->getCellXfCollection()[$i];
			$fontId = 0;
			$fillId = 0;
			$alignmentId = 0;
			$borderId = 0;
			$nfId = 0;
			$quotePrefix = 0;

			if ($defaultFontHashCode !== $cellXf->getFont()->getHashCode()) {
				$fontId = $fontHashTable->getIndexForHashCode($cellXf->getFont()->getHashCode());
			}
			if ($defaultFillHashCode !== $cellXf->getFill()->getHashCode()) {
				$fillId = $fillHashTable->getIndexForHashCode($cellXf->getFill()->getHashCode());
			}
			if ($defaultAlignmentHashCode !== $cellXf->getAlignment()->getHashCode()) {
				$alignmentId = $alignmentHashTable->getIndexForHashCode($cellXf->getAlignment()->getHashCode());
			}
			if ($defaultBordersHashCode !== $cellXf->getBorders()->getHashCode()) {
				// remove diagonal borders
				$borders = clone $cellXf->getBorders();
				$borders->getDiagonal()->setBorderStyle();
				$borders->getDiagonal()->setColor(new PHPExcel_Style_Color());
				$borders->setDiagonalDirection();
				$borderHashTable->add($borders);

				// replace original borders with non-diagonal borders
				$borderId = $borderHashTable->getIndexForHashCode($borders->getHashCode());
			}
			if ($defaultNumberFormatHashCode !== $cellXf->getNumberFormat()->getHashCode()) {
				$nfId = $numFmtHashTable->getIndexForHashCode($cellXf->getNumberFormat()->getHashCode());
			}
			if ($defaultQuotePrefix !== $cellXf->getQuotePrefix()) {
				$quotePrefix = $cellXf->getQuotePrefix() ? 1 : 0;
			}
			$cellStyle = [0, $fontId, $fillId, $alignmentId, $borderId, $nfId, $quotePrefix];
			$stylesheet['styles'][] = $cellStyle;
		}

		// fonts
		for ($i = 1, $count = $fontHashTable->count(); $i < $count; ++$i) {
			$font = $fontHashTable->getByIndex($i);
			$jFont = $this->fillFont($font, $defaultStyle);
			$stylesheet['fonts'][] = count($jFont) > 0 ? $jFont : null;
		}

		for ($i = 1, $count = $fillHashTable->count(); $i < $count; ++$i) {
			$fill = $fillHashTable->getByIndex($i);
			$jFill = array();
			// Background color
			// ref: ISO/IEC-29500-1, 18.8.32 patternFill
			$bgColor = 'FFFFFF';
			if ($fill->getFillType() === PHPExcel_Style_Fill::FILL_SOLID) {
				$bgColor = $fill->getStartColor()->getRGB();
			} else if ($fill->getFillType() !== PHPExcel_Style_Fill::FILL_NONE) {
				$bgColor = $fill->getEndColor()->getRGB();
			}
			if ($bgColor !== 'FFFFFF') {
				$jFill['bg'] = $bgColor;
			}
			$stylesheet['fills'][] = count($jFill) > 0 ? $jFill : null;
		}

		for ($i = 1, $count = $alignmentHashTable->count(); $i < $count; ++$i) {
			$alignment = $alignmentHashTable->getByIndex($i);
			$jAlignment = array();

			switch ($alignment->getHorizontal()) {
			case PHPExcel_Style_Alignment::HORIZONTAL_LEFT :
				$jAlignment['h'] = 'left';
				break;
			case PHPExcel_Style_Alignment::HORIZONTAL_JUSTIFY :
			case PHPExcel_Style_Alignment::HORIZONTAL_CENTER_CONTINUOUS :
			case PHPExcel_Style_Alignment::HORIZONTAL_CENTER :
				$jAlignment['h'] = 'center';
				break;
			case PHPExcel_Style_Alignment::HORIZONTAL_RIGHT :
				$jAlignment['h'] = 'right';
				break;
			}

			switch ($alignment->getVertical()) {
			case PHPExcel_Style_Alignment::VERTICAL_TOP :
				$jAlignment['v'] = 'top';
				break;
			case PHPExcel_Style_Alignment::VERTICAL_JUSTIFY :
			case PHPExcel_Style_Alignment::VERTICAL_CENTER :
				$jAlignment['v'] = 'middle';
				break;
			case PHPExcel_Style_Alignment::VERTICAL_BOTTOM :
				break;
			}

			if ($alignment->getWrapText() === TRUE) {
				$jAlignment['w'] = 'wrap';
			}

			$stylesheet['alignments'][] = count($jAlignment) > 0 ? $jAlignment : null;
		}

		$borderMethods = ['getTop', 'getRight', 'getBottom', 'getLeft'];
		for ($i = 1, $count = $borderHashTable->count(); $i < $count; ++$i) {
			$borders = $borderHashTable->getByIndex($i);
			$jBorder = array();

			foreach($borderMethods as $idx => &$method) {
				$borderSide = $borders->$method();
				if (PHPExcel_Style_Border::BORDER_NONE !== $borderSide->getBorderStyle()) {
					$jBorder[$idx] = '#'.$borderSide->getColor()->getRGB();
				}
			}

			if (empty($jBorder)) {
				$jBorder = null;
			} else {
				$jBorder = array_replace([null, null, null, null], $jBorder);
			}
			$stylesheet['borders'][] = array('b' => $jBorder);
		}

		for ($i = 1, $count = $numFmtHashTable->count(); $i < $count; ++$i) {
			$numFmt = $numFmtHashTable->getByIndex($i);
			$fc = $numFmt->getFormatCode();
			$stylesheet['numberFormats'][] = array('fc' => $fc);
		}

		if (!file_put_contents($folder.'/style', json_encode($stylesheet))) {
			OnError ( "Failed to save index file", __FILE__, __LINE__ );
		}
	}

	private function fillFont($font, $defaultStyle) {
			$jFont = array();
			$fontSize = $font->getSize();
			$fontColor = $font->getColor()->getRGB();
			if (!is_null($fontSize) && $defaultStyle->getFont()->getSize() !== $fontSize && $fontSize !== "10") {
				$jFont['sz'] = intval($fontSize);
			}
			if ($font->getBold()) {
				$jFont['b'] = TRUE;
			}
			if ($font->getItalic()) {
				$jFont['i'] = TRUE;
			}
			if ($font->getStrikethrough()) {
				$jFont['s'] = TRUE;
			}
			if ($font->getUnderline() && $font->getUnderline() != PHPExcel_Style_Font::UNDERLINE_NONE) {
				$jFont['u'] = TRUE;
			}
			if (!is_null($fontColor) && $fontColor !== "000000") {
				$jFont['c'] = $fontColor;
			}
			return $jFont;
	}

	private function initSheet(&$sheet) {
		$colCount = PHPExcel_Cell::columnIndexFromString ( $sheet->getHighestColumn () );
		$rowCount = intval($sheet->getHighestRow ());

		$this->_jsSheet = array(
			'cells' => new stdClass(),
			'colCount' => max($colCount, DEFAULT_COL_COUNT),
			'rowCount' => max($rowCount, DEFAULT_ROW_COUNT),
			'rows' => new stdClass(),
			'cols' => new stdClass(),
			'fixedColumnLeft' => 0,
			'fixedRowTop' => 0,
			'mergeCells' => []
		);

		$topLeftCell = $sheet->getFreezePane();
		if (($topLeftCell !== '') && ($topLeftCell != 'A1')) {
			$xSplit = $ySplit = 0;
			list($xSplit, $ySplit) = PHPExcel_Cell::coordinateFromString(strtoupper($topLeftCell));
			$xSplit = PHPExcel_Cell::columnIndexFromString($xSplit);
			if ($xSplit > 1) {
				$this->_jsSheet['fixedColumnLeft'] = $xSplit - 1;
			}
			if ($ySplit > 1) {
				$this->_jsSheet['fixedRowTop'] = $ySplit - 1;
			}
		}
	}

	private function fillProps($sheet, $outfolder, $index) {
		$sheetState = $sheet->getSheetState();
		$props = array();
		if ($sheetState !== PHPExcel_Worksheet::SHEETSTATE_VISIBLE) {
			$props['hidden'] = true;
		}
		if ($sheet->isTabColorSet()) {
			$props['tabColor'] = $sheet->getTabColor()->getRGB();
		}
		if (count($props) > 0) {
			file_put_contents($outfolder.'/sh_'.$index.'.props', json_encode($props));
		}
	}

	private function rangeToArray(&$range) {
		list ( $cells, ) = PHPExcel_Cell::splitRange ( $range );
		$first = $cells [0];
		$last = $cells [1];

		list ( $fc, $fr ) = PHPExcel_Cell::coordinateFromString(strtoupper($first));
		$fc = PHPExcel_Cell::columnIndexFromString ( $fc ) - 1;
		$fr --;

		list ( $lc, $lr ) = PHPExcel_Cell::coordinateFromString(strtoupper($last));
		$lc = PHPExcel_Cell::columnIndexFromString ( $lc ) - 1;
		$lr --;
		return array($fr, $fc, $lr, $lc);
	}

	private function fillMergeCells(&$sheet) {
		foreach ( $sheet->getMergeCells () as &$cells ) {
			$info = $this->rangeToArray($cells);
			$this->_jsSheet['mergeCells'][] = $info;
		}
	}

	private function fillDimension(&$sheet) {
		// Calculate col widths and hidden col
		$sheet->calculateColumnWidths();

		// find global
		$globalXf = 0;
		foreach ($sheet->getColumnDimensions() as &$columnDimension) {
			if ($columnDimension->getMax() === 16384) {
				$xfIndex = $columnDimension->getXfIndex();
				if ($xfIndex > 0) {
					$globalXf = $xfIndex;
					break;
				}
			}
		}

		foreach ($sheet->getColumnDimensions() as &$columnDimension) {
			if ($columnDimension->getMax() === 16384) {
				continue;
			}
			$colInfo = array();
			$xfIndex = $columnDimension->getXfIndex();
			$width = $columnDimension->getWidth();
			if ($width !== - 1) {
				$width = $width / COL_WIDTH_RATIO;
				$width = min(max($width, MIN_COL_WIDTH), MAX_COL_WIDTH);
				$colInfo['w'] = round($width);
			}
			if ($columnDimension->getVisible() === FALSE) {
				$colInfo['hidden'] = TRUE;
			}
			if ($xfIndex !== 0) {
				$colInfo['s'] = $xfIndex;
			}

			$column = PHPExcel_Cell::columnIndexFromString ( $columnDimension->getColumnIndex () ) - 1;
			if (count($colInfo) > 0) {
				$this->_jsSheet['cols']->$column = $colInfo;
			}
		}

		if ($globalXf > 0) {
			$this->_jsSheet['global'] = array('s' => $globalXf);
		}

		// Calculate row heights and hidden row
		foreach ( $sheet->getRowDimensions () as &$rowDimension ) {
			$height = $rowDimension->getRowHeight();
			$xfIndex = $rowDimension->getXfIndex();
			$rowInfo = array();
			if ($height != - 1) {
				$height = $height / ROW_HEIGHT_RATIO;
				$height = min(max($height, MIN_ROW_HEIGHT), MAX_ROW_HEIGHT);
				$rowInfo['h'] = round($height);
			}
			if ($rowDimension->getVisible() === FALSE) {
				$rowInfo['hidden'] = TRUE;
			}
			if (!is_null($xfIndex) && $xfIndex !== 0) {
				$rowInfo['s'] = $xfIndex;
			}

			if (count($rowInfo) > 0) {
				$row = $rowDimension->getRowIndex() - 1;
				$this->_jsSheet['rows']->$row = $rowInfo;
			}
		}
	}

	private function fillSheet(&$sheet, &$jsIndex) {
		$this->fillDimension($sheet);

		foreach ($sheet->getCellCollection(false) as &$cellID) {
			$this->fillCell($sheet, $sheet->getCell($cellID), $jsIndex);
		}

		$this->fillNotes($sheet);
	}

	private function fillArray(&$arr) {
		$newarr = array ();
		if (empty ( $arr )) {
			return;
		}
		$maxindex = max ( array_keys ( $arr ) );
		for($i = 0; $i <= $maxindex; $i ++) {
			if (! array_key_exists ( $i, $arr )) {
				$newarr [$i] = null;
			} else {
				$newarr [$i] = $arr [$i];
			}
		}
		$arr = $newarr;
	}

	private function fillNotes(&$sheet) {
		$notes = $sheet->getComments();
		foreach ($notes as $key => &$note) {
			$text = $note->getText()->getPlainText();
			if (empty($text)) {
				continue;
			}
			list($col, $row) = PHPExcel_Cell::coordinateFromString(strtoupper($key));
			$col = PHPExcel_Cell::columnIndexFromString($col) - 1;
			$row--;
			$cell = &$this->getCellRef($row, $col);
			// if cell is empty, it would be a empty stdClass
			if (is_array($cell)) {
				$cell['n'] = $text;
			} else {
				$cell->n = $text;
			}
		}
	}

	private function &getCellRef($row, $col) {
		$cells = &$this->_jsSheet['cells'];
		if (!isset($cells->$row)) {
			$cells->$row = new stdClass();
		}
		if (!isset($cells->$row->$col)) {
			$cells->$row->$col = array();
		}
		return $cells->$row->$col;
	}

	private function fillCell(&$sheet, &$cell, &$jsIndex) {
		$row = $cell->getRow() - 1;
		$col = PHPExcel_Cell::columnIndexFromString($cell->getColumn()) - 1;
		$c = array();
		if ($cell->getXfIndex() !== 0) {
			$c['s'] = $cell->getXfIndex();
		}

		switch ($cell->getDataType()) {
			case PHPExcel_Cell_DataType::TYPE_BOOL:
				$c['v'] = (bool) $cell->getValue();
				break;
			case PHPExcel_Cell_DataType::TYPE_NUMERIC:
				$c['v'] = $cell->getValue();
				if (PHPExcel_Shared_Date::getExcelCalendar() === PHPExcel_Shared_Date::CALENDAR_MAC_1904) {
					if ((is_integer($c['v']) || is_float($c['v'])) &&
							$c['v'] >= 1 && PHPExcel_Shared_Date::isDateTime($cell)) {
						$c['v'] += 1462;
					}
				}
				break;
			case PHPExcel_Cell_DataType::TYPE_STRING:
				$text = $cell->getValue();
				if ($text instanceof PHPExcel_RichText) {
					$c['v'] = $this->fillRichText($text, $sheet->getParent()->getDefaultStyle());
				} else if (!empty($text)) { // empty text
					$c['v'] = $text;
				}
				break;
			case PHPExcel_Cell_DataType::TYPE_FORMULA:
				// SpreadSheet #309
				$c['v'] = str_replace("_xlfn.", "", $cell->getValue ());
				break;
			case PHPExcel_Cell_DataType::TYPE_ERROR:
				$c['v'] = PHPExcel_Cell_DataType::checkErrorCode($cell);
				break;
			case PHPExcel_Cell_DataType::TYPE_INLINE:
			case PHPExcel_Cell_DataType::TYPE_NULL:
				break;
		}

		// hyperlink
		if ($cell->hasHyperlink() && !(preg_match('/=\s*HYPERLINK/i', $c['v']))) {
			$link = '';
			if ($cell->getHyperlink()->isInternal()) {
				preg_match("/^sheet:\/\/('([^']|'')+'|[^!#]+)!/", $cell->getHyperlink()->getUrl(), $matches);
				if (count($matches)) {
					$sheetName = $matches[1];
					if ($sheetName[0] === "'") {
							$sheetName = str_replace("''", "'", substr($sheetName, 1, -1));
					}
					foreach($jsIndex['sheets'] as $key => $val) {
						if ($val['title'] === $sheetName) {
							$link = '#tid=' . RemoveSheetPrefix($key);
							break;
						}
					}
				}
			} else {
				$link = $cell->getHyperlink()->getUrl();
			}
			if ($link) {
				$tooltip = $cell->getHyperlink()->getTooltip();
				$c['v'] = sprintf('=HYPERLINK("%s", "%s")',
					$link,
					str_replace('"', '""', empty($tooltip) ? $cell->getValue() : $tooltip));
			}
		}

		$cell = &$this->getCellRef($row, $col);
		// force empty cell encoded to {}
		$cell = count($c) > 0 ? $c : new stdClass();
	}

	private function fillRichText($richText, $defaultStyle) {
		$textRuns = [];
		foreach ($richText->getRichTextElements() as $text) {
			if (empty($text->getText())) {
				continue;
			}
			$t = array(
				'tx' => $text->getText(),
			);
			$font = $text->getFont();
			if ($font !== null) {
				$fontConfig = $this->fillFont($font, $defaultStyle);
				if (count($fontConfig) > 0) {
					$t['s'] = $fontConfig;
				}
			}
			$textRuns[] = $t;
		}
		return count($textRuns) > 0 ? array( 't' => 'r', 'v' => $textRuns ) : '';
	}

	private function randomString() {
		$characters = '0123456789abcdef';
		$charactersLength = strlen($characters);
		$randomString = '';
		for ($i = 0; $i < 16; $i++) {
			$randomString .= $characters[rand(0, $charactersLength - 1)];
		}
		return $randomString;
	}

	private function fillFilter(&$sheet) {
		$autoFilter = $sheet->getAutoFilter();
		if (empty($autoFilter) || empty($autoFilter->getRange())) {
			return;
		}
		$jsFilter = new stdClass();
		list($fr, $fc, $lr, $lc) = $this->rangeToArray(str_replace('$', '', $autoFilter->getRange()));
		$jsFilter->range = [$fr, $fc, $lr, $lc];
		$jsFilter->enabled = true;
		$jsFilter->id = $this->randomString();
		$jsFilter->filters = array();
		foreach($autoFilter->getColumns() as $index => $columnFilter) {
			$colIndex = PHPExcel_Cell::columnIndexFromString($index) - $fc - 1;
			$jsCondition = new stdClass();
			switch($columnFilter->getFilterType()) {
				case PHPExcel_Worksheet_AutoFilter_Column::AUTOFILTER_FILTERTYPE_FILTER:
					$jsCondition->type = 'value';
					$jsCondition->value = array();
					foreach($columnFilter->getRules() as $rIndex => $rule) {
						array_push($jsCondition->value, $rule->getValue());
					}
					break;
				case PHPExcel_Worksheet_AutoFilter_Column::AUTOFILTER_FILTERTYPE_CUSTOMFILTER:
					if ($columnFilter->getJoin() === PHPExcel_Worksheet_AutoFilter_Column::AUTOFILTER_COLUMN_JOIN_AND) {
						$jsCondition->and = true;
					}
					$jsCondition->type = 'custom';
					$jsCondition->operator = array();
					$jsCondition->value = array();
					$ruleCount = count($columnFilter->getRules());
					foreach($columnFilter->getRules() as $rIndex => $rule) {
						$operator = null;
						$value = $rule->getValue();
						if (is_numeric($value)) {
							$value = floatval($value);
						}
						switch($rule->getOperator()) {
							case PHPExcel_Worksheet_AutoFilter_Column_Rule::AUTOFILTER_COLUMN_RULE_GREATERTHAN:
								$operator = 'greaterThan';
								break;
							case PHPExcel_Worksheet_AutoFilter_Column_Rule::AUTOFILTER_COLUMN_RULE_GREATERTHANOREQUAL:
								$operator = 'greaterThanOrEqual';
								break;
							case PHPExcel_Worksheet_AutoFilter_Column_Rule::AUTOFILTER_COLUMN_RULE_LESSTHAN:
								$operator = 'lessThan';
								break;
							case PHPExcel_Worksheet_AutoFilter_Column_Rule::AUTOFILTER_COLUMN_RULE_LESSTHANOREQUAL:
								$operator = 'lessThanOrEqual';
								break;
							case PHPExcel_Worksheet_AutoFilter_Column_Rule::AUTOFILTER_COLUMN_RULE_EQUAL:
								$operator = 'equal';
								if (is_string($value) && $ruleCount === 1) {
									$len = strlen($value);
									if ($len > 1 && $value[0] === '*' && $value[$len - 1] === '*') {
										$operator = 'containsTexts';
										$value = substr($value, 1, $len - 2);
									} else if ($len > 1 && $value[0] === '*' && $value[$len - 1] !== '*') {
										$operator = 'endsWith';
										$value = substr($value, 1, $len - 1);
									} else if ($len > 1 && $value[0] !== '*' && $value[$len - 1] === '*') {
										$operator = 'beginsWith';
										$value = substr($value, 0, $len - 1);
									} else {
										$operator = 'exactTexts';
									}
									$jsCondition->type = 'text';
								}
								break;
							case PHPExcel_Worksheet_AutoFilter_Column_Rule::AUTOFILTER_COLUMN_RULE_NOTEQUAL:
								$operator = 'notEqual';
								if (is_string($value) && $ruleCount === 1) {
									$len = strlen($value);
									if ($len > 1 && $value[0] === '*' && $value[$len - 1] === '*') {
										$operator = 'notContainsTexts';
										$value = substr($value, 1, $len - 2);
										$jsCondition->type = 'text';
									}
								}
								break;
						}
						if (!is_null($operator) && !is_null($value)) {
							array_push($jsCondition->operator, $operator);
							array_push($jsCondition->value, $value);
						}
					}
					break;
			}
			if (property_exists($jsCondition, 'type')) {
				$jsFilter->filters[$colIndex] = $jsCondition;
			}
		}
		$this->fillArray($jsFilter->filters);

		$this->_jsSheet['filter'] = $jsFilter;
	}

	private function fillValidation(&$sheet) {
		$validationCollection = $sheet->getDataValidationCollection();
		$jsValidations = new stdClass();
		foreach($validationCollection as $sqref => $validation) {
			$jsValidation = new stdClass();
			switch($validation->getType()) {
				case PHPExcel_Cell_DataValidation::TYPE_CUSTOM:
					$jsValidation->type = 'custom';
					$jsValidation->operator = 'custom';
					break;
				case PHPExcel_Cell_DataValidation::TYPE_DATE:
					$jsValidation->type = 'date';
					break;
				case PHPExcel_Cell_DataValidation::TYPE_DECIMAL:
				case PHPExcel_Cell_DataValidation::TYPE_WHOLE:
					$jsValidation->type = 'decimal';
					break;
				case PHPExcel_Cell_DataValidation::TYPE_LIST:
					$jsValidation->type = 'list';
					break;
				case PHPExcel_Cell_DataValidation::TYPE_NONE:
				case PHPExcel_Cell_DataValidation::TYPE_TEXTLENGTH:
				case PHPExcel_Cell_DataValidation::TYPE_TIME:
				default:
					break;
			}
			if (empty($jsValidation->type)) {
				continue;
			}
			if (!empty($validation->getOperator())) {
				switch($validation->getOperator()) {
					case PHPExcel_Cell_DataValidation::OPERATOR_BETWEEN:
						$jsValidation->operator = 'between';
						break;
					case PHPExcel_Cell_DataValidation::OPERATOR_NOTBETWEEN:
						$jsValidation->operator = 'notBetween';
						break;
					case PHPExcel_Cell_DataValidation::OPERATOR_EQUAL:
						$jsValidation->operator = 'equal';
						break;
					case PHPExcel_Cell_DataValidation::OPERATOR_NOTEQUAL:
						$jsValidation->operator = 'notEqual';
						break;
					case PHPExcel_Cell_DataValidation::OPERATOR_GREATERTHAN:
						$jsValidation->operator = 'greaterThan';
						break;
					case PHPExcel_Cell_DataValidation::OPERATOR_GREATERTHANOREQUAL:
						$jsValidation->operator = 'greaterThanOrEqual';
						break;
					case PHPExcel_Cell_DataValidation::OPERATOR_LESSTHAN:
						$jsValidation->operator = 'lessThan';
						break;
					case PHPExcel_Cell_DataValidation::OPERATOR_LESSTHANOREQUAL:
						$jsValidation->operator = 'lessThanOrEqual';
						break;
					default:
						break;
				}
			}
			if ($validation->getShowErrorMessage() && $validation->getErrorStyle() === PHPExcel_Cell_DataValidation::STYLE_STOP) {
				$jsValidation->reject = true;
			} else {
				$jsValidation->reject = false;
			}
			if (!empty($validation->getPromptTitle()) || !empty($validation->getPrompt())) {
				if (!empty($validation->getPromptTitle()) && !empty($validation->getPrompt())) {
					$jsValidation->message = $validation->getPromptTitle() . ' - ' . $validation->getPrompt();
				} else {
					$jsValidation->message = $validation->getPromptTitle() . $validation->getPrompt();
				}
			} else if (!empty($validation->getErrorTitle()) || !empty($validation->getError())) {
				if (!empty($validation->getErrorTitle()) && !empty($validation->getError())) {
					$jsValidation->message = $validation->getErrorTitle() . ' - ' . $validation->getError();
				} else {
					$jsValidation->message = $validation->getErrorTitle() . $validation->getError();
				}
			} else {
				$jsValidation->message = '';
			}
			if ('list' === $jsValidation->type) {
				$f1 = $validation->getFormula1();
				if ('"' === $f1[0] && '"' === $f1[strlen($f1) - 1]) {
					$jsValidation->operator = 'list';
					$f1 = substr($f1, 1, -1);
				} else {
					$jsValidation->operator = 'listRange';
					$namedRange = $sheet->getParent()->getNamedRange($f1);
					if (!is_null($namedRange)) {
						$f1 = $namedRange->getRawValue();
					}
					$f1 = str_replace('$', '', $f1);
				}
				$jsValidation->f1 = $f1;
			} else if (('decimal' === $jsValidation->type || 'date' === $jsValidation->type) && is_numeric($validation->getFormula1())){
				$jsValidation->f1 = floatval($validation->getFormula1());
			} else {
				$jsValidation->f1 = $validation->getFormula1();
			}
			if (!empty($validation->getFormula2())) {
				if (('decimal' === $jsValidation->type || 'date' === $jsValidation->type) && is_numeric($validation->getFormula2())) {
					$jsValidation->f2 = floatval($validation->getFormula2());
				} else {
					$jsValidation->f2 = $validation->getFormula2();
				}
			}
			$jsValidation->ranges = explode(' ', $sqref);
			$jsValidation->validId = $this->randomString();
			$jsValidations->{$jsValidation->validId} = $jsValidation;
		}
		$this->_jsSheet['validation'] = $jsValidations;
	}

	private function fillCf(&$sheet) {
		$collection = array();
		foreach ($sheet->getConditionalStylesCollection() as $sqref => $conditionals) {
			foreach ($conditionals as $conditional) {
				$jsCf = new stdClass();
				$jsCf->cfId = $this->randomString();
				$jsCf->type = $conditional->getConditionType();
				if (!empty($conditional->getOperatorType())) {
					$jsCf->op = $conditional->getOperatorType();
				}
				if (!empty($conditional->getText())) {
					$jsCf->text = $conditional->getText();
				}
				$conds = $conditional->getConditions();
				$jsCf->f1 = parseFormulaField($conds[0]);
				if (count($conds) > 1) {
					$jsCf->f2 = parseFormulaField($conds[1]);
				}
				$jsCf->ranges = explode(' ', $sqref);

				$jsStyle = new stdClass();
				$style = $conditional->getStyle();

				if ($style->getFont()->getBold()) {
					$jsStyle->bold = true;
				}
				if ($style->getFont()->getItalic()) {
					$jsStyle->italic = true;
				}
				if ($style->getFont()->getStrikethrough()) {
					$jsStyle->strike = true;
				}

				// For identical behavior as cell style
				if ($style->getFont()->getUnderline() && $style->getFont()->getUnderline() != PHPExcel_Style_Font::UNDERLINE_NONE) {
					$jsStyle->underline = true;
				}

				$color = $style->getFont()->getColor()->getRGB();
				if ($color) {
					$jsStyle->color = $color;
				}
				$bgColor = $style->getFill()->getEndColor()->getRGB();
				if ($bgColor && $bgColor != 'FFFFFF') {
					$jsStyle->bgColor = $bgColor;
				}
				$jsCf->style = $jsStyle;

				$collection[$conditional->getPriority()] = $jsCf;
			}
		}

		ksort($collection);
		$jsCfs = array();
		foreach($collection as $jsCf) {
			$jsCfs[] = $jsCf;
		}
		$this->_jsSheet['cfs'] = $jsCfs;
	}

	private function getColumnInfo($col) {
		return isset($this->_jsSheet['cols']->$col) ? $this->_jsSheet['cols']->$col : null;
	}

	private function getRowInfo($row) {
		return isset($this->_jsSheet['rows']->$row) ? $this->_jsSheet['rows']->$row : null;
	}

	private function getColToX($col, $offsetX) {
		$x = 0;
		for ($i = 0; $i < $col; ++$i) {
			$colInfo = $this->getColumnInfo($i);
			if (is_null($colInfo)) {
				$x += COL_WIDTH;
			} else if (isset($colInfo['hidden']) && $colInfo['hidden'] === TRUE) {
				continue;
			} else if (isset($colInfo['w']) && $colInfo['w'] > 0) {
				$x += $colInfo['w'];
			} else {
				$x += COL_WIDTH;
			}
		}
		return $x + $offsetX;
	}

	private function getRowToY($row, $offsetY) {
		$y = 0;
		for ($i = 0; $i < $row; ++$i) {
			$rowInfo = $this->getRowInfo($i);
			if (is_null($rowInfo)) {
				$y += ROW_HEIGHT;
			} else if (isset($rowInfo['hidden']) && $rowInfo['hidden'] === TRUE) {
				continue;
			} else if (isset($rowInfo['h']) && $rowInfo['h'] > 0) {
				$y += $rowInfo['h'];
			} else {
				$y += ROW_HEIGHT;
			}
		}
		return $y + $offsetY;
	}

	private function fillImages(&$sheet, $outfolder, $index) {
		if (count($sheet->getDrawingCollection()) === 0) {
			return;
		}
		$image_folder = $outfolder . "/images";
		if (!file_exists($image_folder) && !mkdir($image_folder, 0770, true)) {
			OnError("Failed to mkdir. " . $image_folder, __FILE__, __LINE__);
		}
		$zip_tmp_folder = $outfolder . "/$index/unzip";
		if (!file_exists($zip_tmp_folder) && !mkdir($zip_tmp_folder, 0770, true)) {
			OnError("Failed to mkdir. " . $zip_tmp_folder, __FILE__, __LINE__);
		}

		foreach ($sheet->getDrawingCollection() as &$drawing) {
			$image_id = uniqid();
			$img = new stdClass ();
			$image_path = $image_folder . '/' . $image_id;
			$result = false;
			if ($drawing instanceof PHPExcel_Worksheet_MemoryDrawing) {
				$result = file_put_contents($image_path, $drawing->getImageResource());
			} else if (strpos($drawing->getPath(), 'zip://') === 0) {
				$path = $drawing->getPath();
				$pos = strrpos($path, '#');
				$filePath = substr($path, 6, $pos - 6);
				$entry = substr($path, $pos + 1);
				$zip = new ZipArchive();
				$res = $zip->open($filePath);
				if ($res === TRUE) {
					$result = $zip->extractTo($zip_tmp_folder, array($entry)) &&
						rename($zip_tmp_folder."/".$entry, $image_path);
					$zip->close();
				}
			} else {
				$result = copy($drawing->getPath(), $image_path);
			}
			if (!$result) {
				if ($drawing instanceof PHPExcel_Worksheet_MemoryDrawing) {
					OnError("Failed write image.");
				} else {
					OnError("Failed to copy a file: " . $drawing->getPath());
				}
				continue;
			}

			$startCoord = PHPExcel_Cell::coordinateFromString(strtoupper($drawing->getCoordinates()));
			$startCoord[0] = PHPExcel_Cell::columnIndexFromString($startCoord[0]);

			$img->id = $image_id;
			$img->type = "img";
			$img->pos = new stdClass;
			$img->pos->left = $this->getColToX($startCoord[0] - 1, $drawing->getOffsetX()) + ROW_HEADER_WIDTH;
			$img->pos->top = $this->getRowToY($startCoord[1] - 1, $drawing->getOffsetY()) + COL_HEADER_HEIGHT;
			if ('twoCellAnchor' === $drawing->getAnchorType()) {
				$endCoord = PHPExcel_Cell::coordinateFromString(strtoupper($drawing->getEndCoordinates()));
				$endCoord[0] = PHPExcel_Cell::columnIndexFromString($endCoord[0]);
				$img->pos->width = $this->getColToX($endCoord[0] - 1, $drawing->getEndOffsetX()) + ROW_HEADER_WIDTH - $img->pos->left;
				$img->pos->height = $this->getRowToY($endCoord[1] - 1, $drawing->getEndOffsetY()) + COL_HEADER_HEIGHT - $img->pos->top;
			} else {
				$img->pos->width = $drawing->getWidth();
				$img->pos->height = $drawing->getHeight();
			}
			$img->rotation = $drawing->getRotation();
			//$drawing->getDescription()
			//$drawing->getName()
			array_push($this->_jsSheet['drawing'], $img);
		}
	}

	private function fillChartTitle($title = null) {
		$jsTitle = new stdClass();
		if (is_null($title)) {
			$jsTitle->text = '';
			return $jsTitle;
		}
		$caption = $title->getCaption();
		if (is_array($caption)) {
			$caption = $caption[0];
		}
		if (is_string($caption)) {
			$jsTitle->text = $caption;
		} else if ($caption instanceof PHPExcel_RichText) {
			$jsTitle->text = $caption->getPlainText();
			$richTextElements = $caption->getRichTextElements();
			if (!is_null($richTextElements[0])) {
				$font = $richTextElements[0]->getFont();
				$jsTitle->textStyle = $this->fillChartTextStyle($font);
			}
		}
		return $jsTitle;
	}

	private function fillChartLegend($legend = null) {
		$jsLegend = new stdClass();
		if (!is_null($legend)) {
			$jsLegend->position = $legend->getPosition();
		}
		return $jsLegend;
	}

	private function fillChartTextStyle($font = null) {
		$jsTextStyle = new stdClass();
		if (!is_null($font)) {
			$jsTextStyle->fontFamily = $font->getName();
			$jsTextStyle->fontSize = $font->getSize();
			$jsTextStyle->italic = $font->getItalic();
			$jsTextStyle->bold = $font->getBold();
			$jsTextStyle->color = '#'.$font->getColor()->getRGB();
		}
		return $jsTextStyle;
	}

	private function fillCharts(&$sheet) {
		$chartCount = $sheet->getChartCount();
		for ($c = 0; $c < $chartCount; $c++) {
			$pChart = $sheet->getChartByIndex($c);
			$jsChart = $this->fillChart($pChart);
			if ($jsChart) {
				$this->_jsSheet['drawing'][] = $jsChart;
			}
		}
	}

	private function fillChart(&$pChart) {
		$chartType = 'bar';
		$barDir = 'col';
		$grouping = 'standard';
		$jsPlotArea = $this->fillChartPlotArea($pChart, $chartType, $barDir, $grouping);
		if (!$jsPlotArea) {
			return false;
		}
		$jsChartObj = new stdClass();
		$jsChartObj->background = '#FFFFFF';
		$jsChartObj->title = $this->fillChartTitle($pChart->getTitle());
		$jsChartObj->subtitle = null;
		$jsChartObj->legend = $this->fillChartLegend($pChart->getLegend());
		$jsChartObj->textStyle = null;
		$jsChartObj->plotArea = $jsPlotArea;
		$jsChartObj->chartType = $chartType;
		$jsChartObj->barDir = $barDir;
		$jsChartObj->grouping = $grouping;

		$jsChart = new stdClass();
		$jsChart->id = $this->randomString();
		$jsChart->chartObj = $jsChartObj;
		$jsChart->pos = new stdClass();

		$coord = PHPExcel_Cell::coordinateFromString(strtoupper($pChart->getTopLeftCell()));
		$coord[0] = PHPExcel_Cell::columnIndexFromString($coord[0]);
		$jsChart->pos->left = $this->getColToX($coord[0] - 1, $pChart->getTopLeftXOffset()) + ROW_HEADER_WIDTH;
		$jsChart->pos->top = $this->getRowToY($coord[1] - 1, $pChart->getTopLeftYOffset()) + COL_HEADER_HEIGHT;
		$coord = PHPExcel_Cell::coordinateFromString(strtoupper($pChart->getBottomRightCell()));
		$coord[0] = PHPExcel_Cell::columnIndexFromString($coord[0]);
		$jsChart->pos->width = $this->getColToX($coord[0] - 1, $pChart->getBottomRightXOffset()) + ROW_HEADER_WIDTH - $jsChart->pos->left;
		$jsChart->pos->height = $this->getRowToY($coord[1] - 1, $pChart->getBottomRightYOffset()) + COL_HEADER_HEIGHT - $jsChart->pos->top;
		$jsChart->type = "chart";

		return $jsChart;
	}

	private function fillChartPlotArea(&$pChart, &$chartType, &$barDir, &$grouping) {
		$plotArea = $pChart->getPlotArea();
		$groupCount = $plotArea->getPlotGroupCount();
		$layout = $plotArea->getLayout();
		$jsSeries = array();
		$firstType = $plotArea->getPlotGroupByIndex(0)->getPlotType();
		$isHybrid = false;
		for ($i = 0; $i < $groupCount; $i++) {
			$plotGroup = $plotArea->getPlotGroupByIndex($i);
			$type = $plotGroup->getPlotType();
			switch($type) {
				case PHPExcel_Chart_DataSeries::TYPE_BARCHART:
					$chartType = 'bar';
					if ($plotGroup->getPlotDirection() === PHPExcel_Chart_DataSeries::DIRECTION_HORIZONTAL) {
						$barDir = 'row';
					}
					break;
				case PHPExcel_Chart_DataSeries::TYPE_LINECHART:
					$chartType = 'line';
					break;
				case PHPExcel_Chart_DataSeries::TYPE_AREACHART:
					$chartType = 'area';
					break;
				case PHPExcel_Chart_DataSeries::TYPE_PIECHART:
				case PHPExcel_Chart_DataSeries::TYPE_DOUGHTNUTCHART:
					$chartType = 'pie';
					break;
				case PHPExcel_Chart_DataSeries::TYPE_SCATTERCHART:
					$chartType = 'scatter';
					break;
			}
			if ($type !== $firstType) {
				$isHybrid = true;
			}
			if ($plotGroup->getPlotGrouping() === PHPExcel_Chart_DataSeries::GROUPING_STACKED) {
				$grouping = 'stacked';
			}
			$jsSeries = array_merge($jsSeries, $this->fillChartSeries($plotGroup, $chartType, $layout));
		}
		if ($isHybrid === true) {
			$chartType = 'hybrid';
		}
		$hasVal = false;
		foreach ($jsSeries as $jsSer) {
			if ($jsSer->val && $jsSer->val !== '#REF!') {
				$hasVal = true;
				break;
			}
		}
		if ($hasVal === false) {
			return false;
		}
		$this->modifySeriesIndex($jsSeries);
		$jsPlotArea = new stdClass();
		$jsPlotArea->xAxis = $this->fillChartAxis($pChart->getXAxisLabel());
		$jsPlotArea->yAxis = $this->fillChartAxis($pChart->getYAxisLabel());
		$jsPlotArea->series = $jsSeries;
		$jsPlotArea->blankAs = $pChart->getDisplayBlanksAs();
		return $jsPlotArea;
	}

	private function fillChartAxis(&$axisTitle) {
		$jsAxis = new stdClass();
		$jsAxis->title = $this->fillChartTitle($axisTitle);
		return $jsAxis;
	}

	private function fillChartSeries(&$plotGroup, &$type, &$layout) {
		$plotSeriesOrder = $plotGroup->getPlotOrder();
		$smooth = $plotGroup->getSmoothLine();
		$jsCat = null;
		$jsSeries = array();
		foreach($plotSeriesOrder as $plotSeriesIndex => $plotSeriesRef) {
			$plotSeriesCatgory = $plotGroup->getPlotCategoryByIndex($plotSeriesIndex);
			$plotSeriesLabel = $plotGroup->getPlotLabelByIndex($plotSeriesIndex);
			$plotSeriesValue = $plotGroup->getPlotValuesByIndex($plotSeriesIndex);

			$jsSer = new stdClass();
			$jsSer->index = $plotSeriesIndex;
			$jsSer->order = $plotSeriesIndex;
			if (!$jsCat && $plotSeriesCatgory && $plotSeriesCatgory->getDataSource()) {
				$jsCat = $plotSeriesCatgory->getDataSource();
			}
			if ($jsCat) {
				$jsSer->cat = RemoveParentheses($jsCat);
			}
			if ($plotSeriesLabel && $plotSeriesLabel->getDataSource()) {
				$jsSer->tx = $plotSeriesLabel->getDataSource();
			}
			if ($plotSeriesValue && $plotSeriesValue->getDataSource()) {
				$jsSer->val = RemoveParentheses($plotSeriesValue->getDataSource());
				$jsSer->type = $type;
				if ($type === 'line' || $type === 'area') {
					$jsSer->marker = new stdClass();
					$jsSer->marker->size = 5;
					$jsSer->marker->symbol = $plotSeriesValue->getPointMarker();
					$jsSer->smooth = $smooth;
				}
				if ($type === 'pie') {
					$jsSer->label = new stdClass();
					$jsSer->label->showVal = (boolean) $layout->getShowVal();
					$jsSer->label->showCat = (boolean) $layout->getShowCatName();
					$jsSer->label->showPercent = (boolean) $layout->getShowPercent();
				}
			}
			if (!empty($jsSer->val)) {
				$jsSeries[] = $jsSer;
			}
		}
		return $jsSeries;
	}

	private function modifySeriesIndex(&$jsSeries) {
		$maxIndex = 0;
		$indexArray = array();
		foreach($jsSeries as &$jsSer) {
			$maxIndex = max($jsSer->index, $maxIndex);
			$indexArray[$jsSer->index] = $jsSer->index;
		}
		if (count($indexArray) === $maxIndex + 1) {
			return;
		}
		$missCount = 0;
		for ($i = 0; $i <= $maxIndex; $i++) {
			if (!isset($indexArray[$i])) {
				$missCount++;
			} else {
				$indexArray[$i] -= $missCount;
			}
		}
		foreach($jsSubCharts as &$jsSubChart) {
			foreach($jsSubChart->series as &$jsSer) {
				$jsSer->index = $indexArray[$jsSer->index];
				$jsSer->order = $jsSer->index;
			}
		}
	}

	public function convertIndex($filepath, $outfolder, $codepage) {
		set_time_limit ( 0 );

		try {
			$objReader = PHPExcel_IOFactory::createReader ( PHPExcel_IOFactory::identify( $filepath ) );
			if ( !empty( $codepage ) ) {
				$objReader->setInputEncoding( $codepage );
			}
			if (method_exists($objReader, 'listWorksheetNames')) {
				$worksheets = $objReader->listWorksheetNames ( $filepath );
			} else {
				$worksheets = $objReader->listWorksheetInfo ( $filepath );
			}
		} catch ( Exception $e ) {
			OnError ( "$e", __FILE__, __LINE__ );
		}
		$this->saveIndex($outfolder, $worksheets);
	}

	public function convertSheet($index, $title, $filepath, $outfolder, $codepage) {
		set_time_limit ( 0 );

		try {
			$type = PHPExcel_IOFactory::identify($filepath);
			$objReader = PHPExcel_IOFactory::createReader($type);
			if ($type === 'OOCalc') {
				$objReader->setDisableStyle(true);
			}
			$objReader->setIncludeCharts(true);
			if ( !empty( $codepage ) ) {
				$objReader->setInputEncoding( $codepage );
			}
			$objReader->setLoadSheetsOnly ( $title );
			$phpExcel = $objReader -> load ( $filepath );
			$sheet = $phpExcel->getActiveSheet();
		} catch ( Exception $e ) {
			try {
				if (!is_null($phpExcel)) {
					$phpExcel->disconnectWorksheets();
				}
				$objReader->setLoadAllSheets();
				$phpExcel = $objReader -> load ( $filepath );
				$sheet = $phpExcel->getSheet($index);
			} catch ( Exception $e ) {
				OnError ( "$e", __FILE__, __LINE__ );
			}
		}

		$jsIndex = json_decode(file_get_contents($outfolder . '/index'), true);

		if ($index === 1) {
			$this->fillStyles($outfolder, $phpExcel);
			$this->fillDefinedNames($outfolder, $phpExcel);
		}
		$this->initSheet($sheet);
		$this->fillMergeCells($sheet);
		$this->fillSheet($sheet, $jsIndex);
		$this->fillFilter ($sheet);
		$this->fillValidation ($sheet);
		$this->fillCf ($sheet);

		$this->_jsSheet['drawing'] = array();
		$this->fillImages ($sheet, $outfolder, $index);
		$this->fillCharts ($sheet);
		$this->fillProps($sheet, $outfolder, $index);

		if (! file_put_contents ( $outfolder . '/sh_' . $index, json_encode ( $this->_jsSheet ) )) {
			OnError ( "Failed to save sheet file", __FILE__, __LINE__ );
		}
	}
}

/**
 * ****************************************************
 * main procedure
 * ****************************************************
 */
function Usage() {
	$PROGRAM = __FILE__;
	echo <<<EOD

Usage:
	$PROGRAM [OPTION]
OPTION:
INPUT:
--type [workbook/sheet]
--index [sheet index]
--name [sheet name]
--file [excel file]
--codepage [iconv codepage]
Output:
--output [output folder path (Required)]

EOD;
}

$longopts = array(
	'type:',
	'index:',
	'name:',
	'file:',
	'codepage:',
	'output:'
);

$cmdOpts = getopt('', $longopts);

$requiredOpts = ['file', 'output', 'type'];
foreach($requiredOpts as $opt) {
	if (!array_key_exists($opt, $cmdOpts)) {
		Usage();
		exit(1);
	}
}

$phpini = parse_ini_file ( '/etc/php/php.ini' );
date_default_timezone_set ( $phpini ["date.timezone"] );

$writer = new PHPExcel_Writer_Snapshot ();
if ('workbook' === $cmdOpts['type']) {
	$writer->convertIndex($cmdOpts['file'], $cmdOpts ['output'] , $cmdOpts['codepage']);
} else {
	$writer->convertSheet (intval($cmdOpts['index']), $cmdOpts ['name'], $cmdOpts ['file'], $cmdOpts ['output'], $cmdOpts ['codepage']);
}
