<?php

namespace PhpOffice\PhpPresentation\Writer\ProseMirror;

use PhpOffice\PhpPresentation\AbstractShape;
use PhpOffice\PhpPresentation\Slide\Background\Color as BgColor;
use PhpOffice\PhpPresentation\Slide\Background\SchemeColor as BgSchemeColor;
use PhpOffice\PhpPresentation\Slide\Background\Image as BgImage;
use PhpOffice\PhpPresentation\Slide\Transition;
use PhpOffice\PhpPresentation\Slide\Animation;
use PhpOffice\PhpPresentation\Style\Border;
use PhpOffice\PhpPresentation\Style\Color;
use PhpOffice\PhpPresentation\Style\Font;
use PhpOffice\PhpPresentation\Style\SchemeColor;
use PhpOffice\PhpPresentation\Shape\Chart;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Area;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Bar;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Line as LineChart;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Pie;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Scatter;
use PhpOffice\PhpPresentation\Shape\Comment;
use PhpOffice\PhpPresentation\Shape\Group;
use PhpOffice\PhpPresentation\Shape\RichText;
use PhpOffice\PhpPresentation\Shape\RichText\Paragraph;
use PhpOffice\PhpPresentation\Shape\Line;
use PhpOffice\PhpPresentation\Shape\Drawing\Gd;
use PhpOffice\PhpPresentation\Shape\Table;
use PhpOffice\PhpPresentation\Shape\Placeholder;

class AbstractSlide extends Node
{
    protected $phpPresentation;
    protected $phpSlide;
    protected $synoComment = array();
    protected $comments = array();
    protected $shapeMap = array();

    public function __construct($phpPresentation)
    {
        parent::__construct($phpPresentation);
        $this->phpPresentation = $phpPresentation;
    }

    public function setData($phpSlide)
    {
        $this->phpSlide = $phpSlide;

        $this->setBackground();
        $this->setContent();
    }

    protected function setTransition()
    {
        $phpTransition = $this->phpSlide->getTransition();
        if (is_null($phpTransition)) {
            return;
        }
        $type = $phpTransition->getTransitionType();
        $duration = $phpTransition->getDuration();
        if (is_null($duration)) {
            $speed = $phpTransition->getSpeed();
            switch ($speed) {
                case Transition::SPEED_FAST:
                    $duration = 500;
                    break;
                case Transition::SPEED_MEDIUM:
                    $duration = 750;
                    break;
                case Transition::SPEED_SLOW:
                    $duration = 1000;
                    break;
            }
        }
        $transition = array();
        switch ($type) {
            case Transition::TRANSITION_FADE:
                $transition['type'] = 'dissolve';
                break;
            case Transition::TRANSITION_FADE_THROUGH_BLACK:
                $transition['type'] = 'fade';
                break;
            case Transition::TRANSITION_PUSH_RIGHT:
                $transition['type'] = 'slide';
                $transition['options'] = array('from' => 'left');
                break;
            case Transition::TRANSITION_PUSH_LEFT:
                $transition['type'] = 'slide';
                $transition['options'] = array('from' => 'right');
                break;
            case Transition::TRANSITION_PUSH_DOWN:
                $transition['type'] = 'slide';
                $transition['options'] = array('from' => 'top');
                break;
            case Transition::TRANSITION_PUSH_UP:
                $transition['type'] = 'slide';
                $transition['options'] = array('from' => 'bottom');
                break;
        }
        if (is_null($transition['type'])) {
            return;
        }
        $transition['duration'] = $duration;
        $this->setAttr('transition', $transition);
    }

    protected function setAnimations()
    {
        $animations = array();
        foreach ($this->phpSlide->getAnimations() as $phpAnimation) {
            $animation = array();
            if (!array_key_exists($phpAnimation->getShape()->getHashCode(), $this->shapeMap)) {
                continue;
            }
            $animation['nodeId'] = $this->shapeMap[$phpAnimation->getShape()->getHashCode()]->id;
            $animation['duration'] = $phpAnimation->getDuration();
            switch ($phpAnimation->getType()) {
                case Animation::ANIMATION_APPEAR:
                    $animation['type'] = 'appear';
                    $animation['options'] = array('entrance' => true);
                    break;
                case Animation::ANIMATION_DISAPPEAR:
                    $animation['type'] = 'appear';
                    $animation['options'] = array('entrance' => false);
                    break;
                case Animation::ANIMATION_FADE_IN:
                    $animation['type'] = 'fade';
                    $animation['options'] = array('entrance' => true);
                    break;
                case Animation::ANIMATION_FADE_OUT:
                    $animation['type'] = 'fade';
                    $animation['options'] = array('entrance' => false);
                    break;
                default:
                    continue 2;
                    break;
            }
            switch ($phpAnimation->getCondition()) {
                case Animation::CONDITION_CLICK:
                    $animation['start'] = 'click';
                    break;
                case Animation::CONDITION_WITH:
                    $animation['start'] = 'with';
                    break;
                case Animation::CONDITION_AFTER:
                    $animation['start'] = 'after';
                    break;
                default:
                    continue 2;
                    break;
            }
            $animations[] = $animation;
        }
        if (!empty($animations)) {
            $this->setAttr('animations', $animations);
        }
    }

    protected function setBackground()
    {
        $phpBackground = $this->phpSlide->getBackground();
        $background = new Background();
        $background->type = $this->type . '_background';

        if ($phpBackground instanceof BgColor || $phpBackground instanceof BgSchemeColor) {
            if ($phpBackground instanceof BgColor) {
                $color = $this->getColor($phpBackground->getColor());
                $opacity = $this->getOpacity($phpBackground->getColor());
            } else {
                $color = $this->getColor($phpBackground->getSchemeColor());
                $opacity = $this->getOpacity($phpBackground->getSchemeColor());
            }
            $background->setAttr('color', $color);
            $background->setAttr('opacity', $opacity);
        } else if ($phpBackground instanceof BgImage) {
            global $synoTempPath;
            $id = rand();

            $imgPath = "{$synoTempPath}{$id}:background";
            $handle = fopen($imgPath, 'wb', false);
            if (false !== $handle) {
                fwrite($handle, file_get_contents($phpBackground->getPath()));
                fclose($handle);
            }
            unlink($phpBackground->getPath());
            $background->setAttr('file_id', "$id");

            if (100 > $phpBackground->getAlpha()) {
                $background->setAttr('opacity', $phpBackground->getAlpha() / 100);
            }
        }
        $this->addContent($background);
    }

    protected function setContent()
    {
        $contentContainer = $this->createContentContainer();

        $phpShapes = $this->phpSlide->getShapeCollection();
        foreach ($phpShapes as $phpShape) {
            if (!$this->isSupportedShape($phpShape)) {
                continue;
            }
            $shape = $this->handleShape($phpShape, $contentContainer);
            if (is_null($shape)) {
                continue;
            }
            $this->shapeMap[$phpShape->getHashCode()] = $shape;
            $contentContainer->addContent($shape);
        }
        $this->addContent($contentContainer);

        $comments = array();
        foreach ($this->comments as $index => $comment) {
            if (!isset($comment->parentIndex)) {
                $comments[$index] = $comment;
            }
        }
        foreach ($this->comments as $index => $comment) {
            if (isset($comment->parentIndex)) {
                $comments[$comment->parentIndex]->replies[] = $comment;
                unset($comment->parentIndex);
            }
        }
        $this->comments = array();
        foreach ($comments as $comment) {
            $this->comments[] = $comment;
        }

        $this->setAttr('synocomment', $this->synoComment);
    }

    protected function createContentContainer()
    {
        $contentContainer = new ContentContainer();
        $contentContainer->type = $this->type . '_content_container';
        return $contentContainer;
    }

    protected function isSupportedShape($phpShape)
    {
        if ($phpShape->isPlaceHolder()) {
            $phpHolder = $phpShape->getPlaceHolder();
            if (!isset($phpHolder)) {
                return false;
            }
            switch ($phpHolder->getType()) {
                case 'title':
                case 'ctrTitle':
                case 'subTitle':
                case 'body':
                case 'sldNum':
                case 'pic':
                    return true;
                default:
                    return false;
            }
        }
        return true;
    }

    public function getColor($color)
    {
        if (is_null($color)) {
            return null;
        }
        if ($color instanceof SchemeColor) {
            $colorName = $color->getValue();
            $schemeColor = $this->getSchemeColor($colorName);
            if (!isset($schemeColor)) {
                return '';
            }
            $rgb = $schemeColor->getRGB();
            if (SchemeColor::TRANSFORM_LUM === $color->getTransform() && is_int($color->getLumMod())) {
                $lumMod = $color->getLumMod() / 100;
                $r = hexdec(substr($rgb, 0, 2));
                $g = hexdec(substr($rgb, 2, 2));
                $b = hexdec(substr($rgb, 4, 2));

                list($h, $s, $l) = $this->rgbToHsl($r, $g, $b);
                if (is_int($color->getLumOff())) {
                    $l = $l * $lumMod + (1 - $lumMod);
                } else {
                    $l = $l * $lumMod;
                }
                list($r, $g, $b) = $this->hslToRgb($h, $s, $l);

                $rgb = str_pad(dechex($r), 2, '0', STR_PAD_LEFT) . str_pad(dechex($g), 2, '0', STR_PAD_LEFT) . str_pad(dechex($b), 2, '0', STR_PAD_LEFT);
            } else if (SchemeColor::TRANSFORM_TINT === $color->getTransform() && is_int($color->getTint())) {
                $tint = $color->getTint() / 100;
                $r = hexdec(substr($rgb, 0, 2));
                $g = hexdec(substr($rgb, 2, 2));
                $b = hexdec(substr($rgb, 4, 2));
                $r = $r + (255 - $r) * (1 - $tint);
                $g = $g + (255 - $g) * (1 - $tint);
                $b = $b + (255 - $b) * (1 - $tint);
                $rgb = str_pad(dechex($r), 2, '0', STR_PAD_LEFT) . str_pad(dechex($g), 2, '0', STR_PAD_LEFT) . str_pad(dechex($b), 2, '0', STR_PAD_LEFT);
            }

            return $rgb;
        } else {
            return $color->getRGB();
        }
        return null;
    }

    public function getOpacity($color)
    {
        if ($color instanceof Color) {
            return $color->getAlpha() / 100;
        } /*else if ($color instanceof SchemeColor) {
            return $color->getAlpha() / 100;
            $colorName = $color->getValue();
            return $this->getSchemeColor($colorName)->getAlpha() / 100;
        }*/
        return 1;
    }

    public function getTextStyles($font, &$styles, $getTypeface = false)
    {
        Utils::getTextStyles($font, $styles, $getTypeface, $this);
    }

    public function getBorderStyles($borders, &$styles)
    {
        $top = $borders->getTop();
        if (Border::LINE_NOTSET !== $top->getLineStyle()) {
            if (Border::LINE_NONE === $top->getLineStyle()) {
                $styles['borderTopWidth'] = '0px';
            } else {
                $styles['borderTopWidth'] = $top->getLineWidth() . 'px';
            }
            $styles['borderTopColor'] = '#' . $this->getColor($top->getColor());
            $styles['borderTopStyle'] = 'solid';
        }

        $right = $borders->getRight();
        if (Border::LINE_NOTSET !== $right->getLineStyle()) {
            if (Border::LINE_NONE === $right->getLineStyle()) {
                $styles['borderRightWidth'] = '0px';
            } else {
                $styles['borderRightWidth'] = $right->getLineWidth() . 'px';
            }
            $styles['borderRightColor'] = '#' . $this->getColor($right->getColor());
            $styles['borderRightStyle'] = 'solid';
        }

        $bottom = $borders->getBottom();
        if (Border::LINE_NOTSET !== $bottom->getLineStyle()) {
            if (Border::LINE_NONE === $bottom->getLineStyle()) {
                $styles['borderBottomWidth'] = '0px';
            } else {
                $styles['borderBottomWidth'] = $bottom->getLineWidth() . 'px';
            }
            $styles['borderBottomColor'] = '#' . $this->getColor($bottom->getColor());
            $styles['borderBottomStyle'] = 'solid';
        }

        $left = $borders->getLeft();
        if (Border::LINE_NOTSET !== $left->getLineStyle()) {
            if (Border::LINE_NONE === $left->getLineStyle()) {
                $styles['borderLeftWidth'] = '0px';
            } else {
                $styles['borderLeftWidth'] = $left->getLineWidth() . 'px';
            }
            $styles['borderLeftColor'] = '#' . $this->getColor($left->getColor());
            $styles['borderLeftStyle'] = 'solid';
        }
    }

    public function styleToMarks(&$styles, &$marks)
    {
        if (!is_array($styles)) {
            return;
        }

        foreach ($styles as $name => $value) {
            $mark = new \stdClass();
            $mark->_ = $name;
            if (is_bool($value) && !$value) {
                continue;
            } else {
                $mark->value = $value;
            }
            array_push($marks, $mark);
        }
    }

    protected function handleShape($phpShape, $container)
    {
        if ($phpShape->isPlaceHolder() && Placeholder::PH_TYPE_PIC !== $phpShape->getPlaceholder()->getType()) {
            return $this->handleHolder($phpShape, $container);
        } else if ($phpShape instanceof Comment) {
            if ($this instanceof Slide) {
                $commentId = Utils::genSynoId();
                $comment = new \stdClass();
                $comment->author = $phpShape->getAuthor()->getName();
                $comment->id = $commentId;
                $comment->timestamp = $phpShape->getDate() * 1000;
                if (!$phpShape->isReply()) {
                    $comment->lastModified = $comment->timestamp;
                    $comment->type = 0;
                    $this->synoComment[] = $commentId;
                } else {
                    $comment->parentIndex = $phpShape->getParentIndex();
                }
                $comment->status = 0;
                $comment->text = $phpShape->getText();
                $comment->replies = array();

                $this->comments[$phpShape->getIndex()] = $comment;
            }
        } else {
            return $this->handleBox($phpShape, $container);
        }
    }

    protected function handleHolder($phpShape, $container)
    {
        $phpHolder = $phpShape->getPlaceHolder();
        switch ($phpHolder->getType()) {
            case 'title':
            case 'ctrTitle':
                $holder = new TitleHolder($this, $container);
                break;
            case 'subTitle':
                $holder = new SubTitleHolder($this, $container);
                break;
            case 'body':
                $holder = new BodyHolder($this, $container);
                break;
            case 'sldNum':
                $holder = new PageNumberHolder($this, $container);
                break;
            default:
                //throw new \Exception("Not supported holder [" . $phpHolder->getType()  . "]");
                return;
        }
        $holder->setData($phpShape);

        return $holder;
    }

    protected function handleBox($phpShape, $container)
    {
        if ($phpShape instanceof RichText) {
            if (true !== Shape::checkShapeSupport($phpShape->getShapeType())) {
                $phpShape->setShapeType('rect');
            }
            if ($phpShape->isTextBox()) {
                $box = new TextBox($this, $container);
            } else {
                $box = new ShapeBox($this, $container);
            }
        } else if ($phpShape instanceof Line) {
            if (true !== Shape::checkShapeSupport($phpShape->getShapeType())) {
                return;
            }
            $box = new LineBox($this, $container);
        } else if ($phpShape instanceof Gd) {
            $box = new ImageBox($this, $container);
        } else if ($phpShape instanceof Group) {
            $box = new ObjectGroup($this, $container);
        } else if ($phpShape instanceof Table) {
            $box = new TableBox($this, $container);
        } else if ($phpShape instanceof Chart) {
            if (!$this->checkChartIsSupported($phpShape)) {
                return;
            }
            $box = new ChartBox($this, $container);
        } else {
            return;
        }
        $box->setData($phpShape);

        return $box;
    }

    private function checkChartIsSupported($phpShape)
    {
        $phpPlotArea = $phpShape->getPlotArea();
        $phpTypes = $phpPlotArea->getTypes();
        foreach ($phpTypes as $phpType) {
            if ($phpType instanceof Area ||
                $phpType instanceof Bar ||
                $phpType instanceof LineChart ||
                $phpType instanceof Pie ||
                $phpType instanceof Scatter) {
                    continue;
            }
            return false;
        }
        if (0 === count($phpTypes)) {
            return false;
        }

        return true;
    }

    public function getPhpSlide()
    {
        return $this->phpSlide;
    }

    private function rgbToHsl($r, $g, $b)
    {
        $r /= 255;
        $g /= 255;
        $b /= 255;

        $max = max($r, $g, $b);
        $min = min($r, $g, $b);

        $l = ( $max + $min ) / 2;
        $d = $max - $min;

        if (0 == $d) {
            return array(0, 0, round($l, 2));
        }

        $s = $d / (1 - abs(2 * $l - 1));
        switch ($max) {
            case $r:
                $h = 60 * fmod((($g - $b) / $d), 6);
                if ($b > $g) {
                    $h += 360;
                }
                break;
            case $g:
                $h = 60 * (($b - $r) / $d + 2);
                break;
            case $b:
                $h = 60 * (($r - $g) / $d + 4);
                break;
        }

        return array(round($h, 2), round($s, 2), round($l, 2));
    }

    private function hslToRgb($h, $s, $l)
    {
        $c = (1 - abs(2 * $l - 1)) * $s;
        $x = $c * (1 - abs(fmod(($h / 60), 2) - 1));
        $m = $l - ($c / 2);

        if ($h < 60) {
            $r = $c;
            $g = $x;
            $b = 0;
        } else if ($h < 120) {
            $r = $x;
            $g = $c;
            $b = 0;
        } else if ($h < 180) {
            $r = 0;
            $g = $c;
            $b = $x;
        } else if ($h < 240) {
            $r = 0;
            $g = $x;
            $b = $c;
        } else if ($h < 300) {
            $r = $x;
            $g = 0;
            $b = $c;
        } else {
            $r = $c;
            $g = 0;
            $b = $x;
        }

        $r = ($r + $m) * 255;
        $g = ($g + $m) * 255;
        $b = ($b + $m) * 255;

        return array(floor($r), floor($g), floor($b));
    }

    public function getTableStyle($styleId)
    {
        return $this->phpPresentation->getTableStyle($styleId);
    }

    public function getDefaultListStyle()
    {
        return $this->phpPresentation->getListStyle();
    }

    public function getComments()
    {
        if ($this instanceof Slide) {
            return $this->comments;
        }
    }
}

?>
