<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Reader\ODText;

use PhpOffice\PhpWord\PhpWord;
use PhpOffice\PhpWord\Shared\XMLReader;

/**
 * Content reader
 *
 * @since 0.10.0
 */
class Content extends AbstractPart
{
    const NUMBER_LIST_MARGIN_BASE_ODT = 1.27;
    const NUMBER_LIST_MARGIN_BASE_DOCX = 810;
    const NUMBER_LIST_MARGIN_OFFSET_ODT = 0.635;
    const NUMBER_LIST_MARGIN_OFFSET_DOCX = 450;

    const BULLET_LIST_MARGIN_BASE_ODT = 1.905;
    const BULLET_LIST_MARGIN_BASE_DOCX = 720;
    const BULLET_LIST_MARGIN_OFFSET_ODT = 0.635;
    const BULLET_LIST_MARGIN_OFFSET_DOCX = 720;

    const BULLET_LIST_MARGIN_HANGING_DOCX = 360;

    private $readMethods = array(
        'text:h' => 'readHeading',
        'text:p' => 'readParagraph',
        'text:list' => 'readList',
        'text:list-item' => 'readListItem',
        'table:table' => 'readTable',
        'text:span' => 'readSpan',
        'text:a' => 'readAnchor',
        'text:s' => 'readSpace',
        'draw:frame' => 'readDraw',
        'text:table-of-content' => 'readToc'
    );

    private $parseStyleMethods = array(
        'style:paragraph-properties' => array('read' => 'readParagraphStyle', 'styleType' => self::STYLE_TYPE_PARAGRAPH),
        'style:text-properties' => array('read' => 'readFontStyle', 'styleType' => self::STYLE_TYPE_FONT),
        'style:table-column-properties' => array('read' => 'readColumnStyle', 'styleType' => self::STYLE_TYPE_COLUMN),
        'style:table-row-properties' => array('read' => 'readRowStyle', 'styleType' => self::STYLE_TYPE_ROW),
        'style:table-cell-properties' => array('read' => 'readCellStyle', 'styleType' => self::STYLE_TYPE_CELL)
    );

    private $styles = array();

    /**
     * Read content.xml.
     *
     * @param \PhpOffice\PhpWord\PhpWord $phpWord
     * @return void
     */
    public function read(PhpWord $phpWord)
    {
        $xmlReader = new XMLReader();
        $xmlReader->getDomFromZip($this->docFile, $this->xmlFile);

        $nodes = $xmlReader->getElements('office:automatic-styles/style:style');
        foreach ($nodes as $node) {
            $name = $xmlReader->getAttribute('style:name', $node);
            $parentName = $xmlReader->getAttribute('style:parent-style-name', $node);

            $styleTypeNodes = $xmlReader->getElements('*', $node);
            foreach ($styleTypeNodes as $styleTypeNode) {
                $parseStyleMethod = $this->parseStyleMethods[$styleTypeNode->nodeName];
                if (!isset($parseStyleMethod)) {
                    continue;
                }
                $funcName = $parseStyleMethod['read'];
                $style = $this->$funcName($xmlReader, $styleTypeNode);
                $this->setStyle($name, $parseStyleMethod['styleType'], $style);
            }
            if (isset($parentName)) {
                $this->styles[$name]['parent'] = $parentName;
            }
        }

        $nodes = $xmlReader->getElements('office:automatic-styles/text:list-style');
        foreach ($nodes as $node) {
            $name = $xmlReader->getAttribute('style:name', $node);
            $numId = floatval(substr($name, 1, 1));
            $style = array('levels' => array(), 'type' => 'hybridMultilevel', 'numId' => $numId);

            $listLvNodes = $xmlReader->getElements('*', $node);
            foreach ($listLvNodes as $listLvNode) {
                $levelId = ((int)$xmlReader->getAttribute('text:level', $listLvNode)) - 1;
                $style['levels'][$levelId] = $this->readListLevelStyle($xmlReader, $listLvNode, $levelId);
            }
            $phpWord->addNumberingStyle("PHPWordList${numId}", $style);
        }

        $nodes = $xmlReader->getElements('office:body/office:text/*');
        if (0 >= $nodes->length) {
            return;
        }

        $section = $phpWord->addSection();
        foreach ($nodes as $node) {
            if ('text:list' == $node->nodeName) {
                $this->readList($xmlReader, $node, $section, -1, null);
            } else {
                $readMethod = $this->readMethods[$node->nodeName];
                if (!isset($readMethod)) {
                    continue;
                }
                $this->$readMethod($xmlReader, $node, $section);
            }
        }
    }

    protected function readListLevelStyle(XMLReader $xmlReader, \DOMElement $domNode, $levelId)
    {
        $level = array();

        $level['level'] = $levelId;
        if ('text:list-level-style-number' === $domNode->nodeName) {
            $level['start'] = $xmlReader->getAttribute('style:num-format', $domNode);
            $level['format'] = 'decimal';
            $text = $xmlReader->getAttribute('text:level', $domNode);
            $level['text'] = "%$text.";
        } else if ('text:list-level-style-bullet' === $domNode->nodeName) {
            $level['start'] = '1';
            $level['format'] = 'bullet';
            $level['text'] = $xmlReader->getAttribute('text:bullet-char', $domNode);
        } else {
            return $level;
        }
        $level['left'] = \PhpOffice\PhpWord\Shared\Converter::cmToTwip($this->parseCm($xmlReader->getAttribute('fo:margin-left', $domNode, 'style:list-level-properties/style:list-level-label-alignment')));
        $level['align'] = 'left';
        $level['hanging'] = self::BULLET_LIST_MARGIN_HANGING_DOCX;

        return $level;
    }

    protected function readChildNodes(XMLReader $xmlReader, \DOMElement $domNode, $parent, $fontStyle, $paragraphStyle)
    {
        $subNodes = $domNode->childNodes;
        if (0 === $subNodes->length) {
            $parent->addText(' ', $fontStyle, $paragraphStyle);
            return;
        }
        foreach ($subNodes as $subNode) {
            //hack for TOC
            if ('text:tab' == $subNode->nodeName && 'text:index-body' == $domNode->parentNode->nodeName) {
                break;
            }

            if ('#text' == $subNode->nodeName) {
                $parent->addText($subNode->nodeValue, $fontStyle, $paragraphStyle);
            } else {
                $readMethod = $this->readMethods[$subNode->nodeName];
                if (!isset($readMethod)) {
                    continue;
                }
                if ('readSpace' === $readMethod) {
                    $this->$readMethod($xmlReader, $subNode, $parent, $fontStyle, $paragraphStyle);
                } else {
                    $this->$readMethod($xmlReader, $subNode, $parent);
                }
            }
        }
    }

    protected function readHeading(XMLReader $xmlReader, \DOMElement $domNode, $parent)
    {
        if ($xmlReader->elementExists('text:bookmark-start', $domNode)) {
            $synohead = $xmlReader->getAttribute('text:name', $domNode, 'text:bookmark-start');
        }
        $styleName = $xmlReader->getAttribute('text:style-name', $domNode);
        $headerName = $styleName;
        $level = 1;

        while (false == strpos($headerName, 'Heading')) {
            if (!isset($this->styles[$headerName]['parent'])) {
                break;
            }
            $headerName = $this->styles[$headerName]['parent'];
        }
        if (preg_match('/Heading_\d+_(\d+)/', $headerName, $matches)) {
            if (2 == count($matches)) {
                $level = $matches[1];
            }
        }

        $paragraphStyle = $this->getLocalAndGlobalStyle($styleName, self::STYLE_TYPE_PARAGRAPH);
        $fontStyle = $this->getLocalAndGlobalStyle($styleName, self::STYLE_TYPE_FONT);

        $titleRun = $parent->addTitleRun($paragraphStyle, $level, $synohead, $parent->getPhpWord());
        $this->readChildNodes($xmlReader, $domNode, $titleRun, $fontStyle, $paragraphStyle);
    }

    protected function readParagraph(XMLReader $xmlReader, \DOMElement $domNode, $parent)
    {
        $styleName = $xmlReader->getAttribute('text:style-name', $domNode);

        $paragraphStyle = $this->getLocalAndGlobalStyle($styleName, self::STYLE_TYPE_PARAGRAPH);
        $fontStyle = $this->getLocalAndGlobalStyle($styleName, self::STYLE_TYPE_FONT);

        //hack for TOC indent
        $parentStyleName = $this->styles[$styleName]['parent'];
        if (isset($parentStyleName) && isset($this->styles[$parentStyleName][self::STYLE_TYPE_PARAGRAPH]['indent'])) {
            $paragraphStyle['indent'] = $this->styles[$parentStyleName][self::STYLE_TYPE_PARAGRAPH]['indent'];
        }

        $textRun = $parent->addTextRun($paragraphStyle);
        $this->readChildNodes($xmlReader, $domNode, $textRun, $fontStyle, $paragraphStyle);
    }

    protected function readSpan(XMLReader $xmlReader, \DOMElement $domNode, $parent)
    {
        $styleName = $xmlReader->getAttribute('text:style-name', $domNode);
        $paragraphStyle = $this->getLocalAndGlobalStyle($styleName, self::STYLE_TYPE_PARAGRAPH);
        $fontStyle = $this->getLocalAndGlobalStyle($styleName, self::STYLE_TYPE_FONT);

        $this->readChildNodes($xmlReader, $domNode, $parent, $fontStyle, $paragraphStyle);
    }

    protected function readAnchor(XMLReader $xmlReader, \DOMElement $domNode, $parent)
    {
        $styleName = $xmlReader->getAttribute('text:style-name', $domNode);
        $paragraphStyle = $this->getLocalAndGlobalStyle($styleName, self::STYLE_TYPE_PARAGRAPH);
        $fontStyle = $this->getLocalAndGlobalStyle($styleName, self::STYLE_TYPE_FONT);
        $target = $xmlReader->getAttribute('xlink:href', $domNode);

        //hack for TOC
        if (false !== strpos($target, '#__RefHeading__') && !$xmlReader->elementExists('text:tab', $domNode)) {
            return;
        }

        $link = $parent->addLink($target, '', $fontStyle, $paragraphStyle, false);
        $this->readChildNodes($xmlReader, $domNode, $link, $fontStyle, $paragraphStyle);
    }

    protected function readSpace(XMLReader $xmlReader, \DOMElement $domNode, $parent, $fontStyle, $paragraphStyle)
    {
        $spaceCount = $xmlReader->getAttribute('text:c', $domNode);
        if (!isset($spaceCount)) {
            $spaceCount = 1;
        }
        $spaces = str_repeat(' ', $spaceCount);
        $parent->addText($spaces, $fontStyle, $paragraphStyle);
    }

    protected function readList(XMLReader $xmlReader, \DOMElement $domNode, $parent, $levelId, $parentListStyleName)
    {
        ++$levelId;

        $listStyleName = $xmlReader->getAttribute('text:style-name', $domNode);
        if (!isset($listStyleName)) {
            $listStyleName = $parentListStyleName;
        } else {
            $numId = floatval(substr($listStyleName, 1, 1));
            $listStyleName = "PHPWordList{$numId}";
        }

        $nodes = $xmlReader->getElements('*', $domNode);
        foreach ($nodes as $node) {
            //child node of list node must be list-item
            if ('text:list-item' != $node->nodeName) {
                continue;
            }
            $this->readListItem($xmlReader, $node, $parent, $levelId, $listStyleName);
        }
    }

    protected function readListItem(XMLReader $xmlReader, \DOMElement $domNode, $parent, $levelId, $listStyleName)
    {
        $nodes = $xmlReader->getElements('*', $domNode);
        foreach ($nodes as $node) {
            //child node of list-item node must not be list-item
            if ('text:list-item' == $node->nodeName) {
                continue;
            }
            if ('text:list' == $node->nodeName) {
                $this->readList($xmlReader, $node, $parent, $levelId, $listStyleName);
            } else {
                $readMethod = $this->readMethods[$node->nodeName];
                if (!isset($readMethod)) {
                    continue;
                }
                $listItemRun = $parent->addListItemRun($levelId, $listStyleName, array());
                $this->$readMethod($xmlReader, $node, $listItemRun);
            }
        }
    }

    protected function readTable(XMLReader $xmlReader, \DOMElement $domNode, $parent)
    {
        //FIXME
        $tblStyle = array();
        $table = $parent->addTable($tblStyle);

        $tblCols = $xmlReader->getElements('table:table-column', $domNode);
        $colgroup = array();
        foreach ($tblCols as $tblCol) {
            $styleName = $xmlReader->getAttribute('table:style-name', $tblCol);
            $colStyle = $this->getStyle($styleName, self::STYLE_TYPE_COLUMN);
            $colWidth = (!empty($colStyle)) ? (int)$colStyle['width'] : '1000';

            $colsRepeated = $xmlReader->getAttribute('table:number-columns-repeated', $tblCol);
            $colsRepeated = $colsRepeated ? (int)$colsRepeated : 1;
            while (0 < $colsRepeated) {
                $colgroup[] = array('width' => $colWidth);
                --$colsRepeated;
            }
        }
        $table->setColgroup($colgroup);

        $cellSpanInfo = array();
        $rowIndex = 0;
        $tblRows = $xmlReader->getElements('table:table-row', $domNode);
        foreach ($tblRows as $tblRow) {
            $styleName = $xmlReader->getAttribute('table:style-name', $tblRow);
            $rowStyle = $this->getStyle($styleName, self::STYLE_TYPE_ROW);
            $rowHeight = (isset($rowStyle)) ? $rowStyle['height'] : null;
            $exactHeight = (isset($rowStyle)) ? $rowStyle['exactHeight'] : false;

            $row = $table->addRow(
                $rowHeight,
                array(
                    'tblHeader' => false,
                    'cantSplit' => false,
                    'exactHeight' => $exactHeight
                )
            );

            $cellIndex = 0;
            $skipCount = 0;
            $tblCells = $xmlReader->getElements('*', $tblRow);
            foreach ($tblCells as $tblCell) {
                if (0 !== $skipCount) {
                    $skipCount--;
                    continue;
                }
                $styleName = $xmlReader->getAttribute('table:style-name', $tblCell);
                $cellStyle = $this->getStyle($styleName, self::STYLE_TYPE_CELL);

                $colSpan = $xmlReader->getAttribute('table:number-columns-spanned', $tblCell);
                if (isset($colSpan)) {
                    $cellStyle['gridSpan'] = $colSpan;
                    $skipCount = (int)$colSpan - 1;
                }
                $rowSpan = $xmlReader->getAttribute('table:number-rows-spanned', $tblCell);
                if (isset($rowSpan)) {
                    $cellStyle['vMerge'] = 'restart';
                }
                if (isset($colSpan) && isset($rowSpan)) {
                    $cellSpanInfo[$cellIndex] = array('endRow' => $rowIndex + (int)$rowSpan - 1, 'colSpan' => $colSpan);
                }
                if ('table:covered-table-cell' === $tblCell->nodeName) {
                    if (isset($cellSpanInfo[$cellIndex]) && $rowIndex <= $cellSpanInfo[$cellIndex]['endRow']) {
                        $colSpan = $cellSpanInfo[$cellIndex]['colSpan'];
                        $cellStyle['gridSpan'] = $colSpan;
                        $skipCount = (int)$colSpan - 1;
                    }
                    $cellStyle['vMerge'] = 'continue';
                }
                $cell = $row->addCell(null, $cellStyle);

                $cellNodes = $xmlReader->getElements('*', $tblCell);
                foreach ($cellNodes as $cellNode) {
                    $readMethod = $this->readMethods[$cellNode->nodeName];
                    if (!isset($readMethod)) {
                        continue;
                    }
                    $this->$readMethod($xmlReader, $cellNode, $cell);
                }
                $cellIndex++;
            }
            $rowIndex++;
        }
    }

    protected function readDraw(XMLReader $xmlReader, \DOMElement $domNode, $parent)
    {
        if ($xmlReader->elementExists('draw:object', $domNode)) {
            $this->readChart($xmlReader, $domNode, $parent);
        } else if ($xmlReader->elementExists('draw:image', $domNode)) {
            $this->readImage($xmlReader, $domNode, $parent);
        }
    }

    protected function readToc(XMLReader $xmlReader, \DOMElement $domNode, $parent)
    {
        $tocTitleNode = $xmlReader->getElement('text:index-body/text:index-title/text:p', $domNode);
        $this->readParagraph($xmlReader, $tocTitleNode, $parent);

        $tocRun = $parent->addTOCRun();

        $tocEntryTplNodes = $xmlReader->getElements('text:table-of-content-source/text:table-of-content-entry-template', $domNode);
        foreach ($tocEntryTplNodes as $tocEntryTplNode) {
            $styleName = $xmlReader->getAttribute('text:style-name', $tocEntryTplNode);
            $level = $xmlReader->getAttribute('text:outline-level', $tocEntryTplNode);
            $this->styles[$styleName] = array(self::STYLE_TYPE_PARAGRAPH => array('indent' => ((int)$level) * 320));
        }

        $tocEntryNodes = $xmlReader->getElements('text:index-body/text:p', $domNode);
        foreach ($tocEntryNodes as $tocEntryNode) {
            $this->readParagraph($xmlReader, $tocEntryNode, $tocRun);
        }
    }

    private function readDimension(XMLReader $xmlReader, $domNode, $convertTo)
    {
        $numberParser = function($value) {
            if (preg_match('/^\d+\.*\d*/', $value, $matches)) {
                return floatval($matches[0]);
            }
            return 0;
        };

        $width = $xmlReader->getAttribute('svg:width', $domNode);
        $height = $xmlReader->getAttribute('svg:height', $domNode);

        if (isset($width) && 0 != ($width = $numberParser($width))) {
            if ('emu' == $convertTo) {
                $width = \PhpOffice\PhpWord\Shared\Converter::cmToEmu($width);
            } else if ('pixel' == $convertTo) {
                $width = \PhpOffice\PhpWord\Shared\Converter::cmToPixel($width);
            }
        } else {
            if ('emu' == $convertTo) {
                $width = \PhpOffice\PhpWord\Shared\Converter::pixelToEmu(520);
            } else {
                $width = 520;
            }
        }
        if (isset($height) && 0 != ($height = $numberParser($height))) {
            if ('emu' == $convertTo) {
                $height = \PhpOffice\PhpWord\Shared\Converter::cmToEmu($height);
            } else if ('pixel' == $convertTo) {
                $height = \PhpOffice\PhpWord\Shared\Converter::cmToPixel($height);
            }
        } else {
            if ('emu' == $convertTo) {
                $height = \PhpOffice\PhpWord\Shared\Converter::pixelToEmu(380);
            } else {
                $height = 380;
            }
        }

        return array(
            'width' => $width,
            'height' => $height
        );
    }

    private function readChart(XMLReader $xmlReader, \DOMElement $domNode, $parent)
    {
        try {
            $dimension = $this->readDimension($xmlReader, $domNode, 'emu');
            $target = $xmlReader->getAttribute('xlink:href', $domNode, 'draw:object');
            $target = str_replace('./', '', $target) . '/content.xml';

            $xmlReader = new XMLReader();
            $xmlReader->getDomFromZip($this->docFile, $target);
            $config = $this->parseChartConfig($xmlReader);
            try {
                $chartType = $config['plotArea']['subCharts'][0]['type'];
            } catch (\Exception $e) {
                throw new \Exception('Fail to read chart: chart type is not defined');
            }
            $chart = $parent->addChart($chartType, array(), array());
            $chart->setSynoChartConfig($config);
            $chart->getStyle()->setWidth($dimension['width']);
            $chart->getStyle()->setHeight($dimension['height']);
        } catch (\Exception $e) {
            print_r($e);
        }
    }

    private function readImage(XMLReader $xmlReader, \DOMElement $domNode, $parent)
    {
        $dimension = $this->readDimension($xmlReader, $domNode, 'pixel');
        $target = $xmlReader->getAttribute('xlink:href', $domNode, 'draw:image');

        $trimPath = str_replace('/', '', $target);
        $name = explode('.', $trimPath)[0];

        $src = "zip://{$this->docFile}#{$target}";
        $style = array(
            'width'        => $dimension['width'],
            'height'       => $dimension['height'],
            'synoFileType' => 'image',
            'synoAlt'      => $name,
            'synoFileId'   => $name
        );
        $parent->addImage($src, $style);
    }

    private function parseChartConfig(XMLReader $xmlReader)
    {
        $config = array();
        $chartNode = $xmlReader->getElement('office:body/office:chart/chart:chart');
        $titleNode = $xmlReader->getElement('chart:title/text:p', $chartNode);
        $config['title'] = (isset($titleNode)) ? $titleNode->nodeValue : '';
        $config['cornerData'] = '';
        $config['plotArea'] = array();
        $config['plotArea']['xAxisTitle'] = $this->getAxisTitle($xmlReader, $chartNode, 'x');
        $config['plotArea']['yAxisTitle'] = $this->getAxisTitle($xmlReader, $chartNode, 'y');
        $config['plotArea']['subCharts'] = array();

        $chartStyle = $this->getChartStyle($xmlReader);
        $chartType = $xmlReader->getAttribute('chart:class', $chartNode);
        $comboChart = $this->checkIsComboChart($xmlReader, $chartNode);

        $chart = array();
        $chart['grouping'] = $chartStyle['grouping'];

        $pieChart = false;
        switch ($chartType) {
            case 'chart:bar':
                $chart['type'] = 'bar';
                $chart['barDir'] = $chartStyle['direction'];
                break;
            case 'chart:line':
                $chart['type'] = 'line';
                break;
            case 'chart:area':
                $chart['type'] = 'area';
                break;
            case 'chart:circle':
                $chart['type'] = 'pie';
                $pieChart = true;
                break;
            case 'chart:scatter':
                $chart['type'] = 'scatter';
                break;
            default:
                break;
        }

        $chart['series'] = $this->parseChartSeries($xmlReader, $chartNode, $pieChart);
        if ($comboChart) {
            $lastItem = array_pop($chart['series']);
            $config['plotArea']['subCharts'][] = $chart;

            $chart = array();
            $chart['type'] = 'line';
            $chart['series'] = array($lastItem);
            $config['plotArea']['subCharts'][] = $chart;
        } else {
            $config['plotArea']['subCharts'][] = $chart;
        }

        return $config;
    }

    private function getAxisTitle(XMLReader $xmlReader, $chartNode, $axis)
    {
        $axisNodes = $xmlReader->getElements('chart:plot-area/chart:axis', $chartNode);

        foreach ($axisNodes as $axisNode) {
            if ($axis != $xmlReader->getAttribute('chart:dimension', $axisNode)) {
                continue;
            }
            if (!$xmlReader->elementExists('chart:title', $axisNode)) {
                return '';
            }
            return $xmlReader->getValue('chart:title/text:p', $axisNode);
        }
    }

    private function getChartStyle(XMLReader $xmlReader)
    {
        $chartStyle = array(
            'grouping' => 'clustered',
            'direction' => 'col'
        );

        $plotAreaNode = $xmlReader->getElement('office:body/office:chart/chart:chart/chart:plot-area');
        if (!isset($plotAreaNode)) {
            return $chartStyle;
        }
        $styleName = $xmlReader->getAttribute('chart:style-name', $plotAreaNode);
        if (!isset($styleName)) {
            return $chartStyle;
        }
        $styleNodes = $xmlReader->getElements('office:automatic-styles/style:style');
        foreach ($styleNodes as $styleNode) {
            if ($styleName != $xmlReader->getAttribute('style:name', $styleNode)) {
                continue;
            }
            if (true == $xmlReader->getAttribute('chart:stacked', $styleNode, 'style:chart-properties')) {
                $chartStyle['grouping'] = 'stacked';
            }
            if (true == $xmlReader->getAttribute('chart:vertical', $styleNode, 'style:chart-properties')) {
                $chartStyle['direction'] = 'row';
            }
            break;
        }
        return $chartStyle;
    }

    private function checkIsComboChart(XMLReader $xmlReader, $chartNode)
    {
        $seriesNodes = $xmlReader->getElements('chart:plot-area/chart:series', $chartNode);
        $barChartExists = false;
        $lineChartExists = false;

        foreach ($seriesNodes as $seriesNode) {
            if ('chart:bar' == $xmlReader->getAttribute('chart:class', $seriesNode)) {
                $barChartExists = true;
            }
            if ('chart:line' == $xmlReader->getAttribute('chart:class', $seriesNode)) {
                $lineChartExists = true;
            }
        }

        return ($barChartExists && $lineChartExists) ? true : false;
    }

    private function parseChartSeries(XMLReader $xmlReader, $chartNode, $pieChart)
    {
        $series = array();
        if (!$xmlReader->elementExists('table:table', $chartNode)) {
            return $series;
        }
        $headerNodes = $xmlReader->getElements('table:table/table:table-header-rows/table:table-row/table:table-cell', $chartNode);
        $catNodes = $xmlReader->getElements('table:table/table:table-rows/table:table-row', $chartNode);
        $headers = array();
        $cats = array();

        foreach($headerNodes as $headerNode) {
            $headers[] = $headerNode;
        }
        foreach ($catNodes as $catNode) {
            $cats[] = $xmlReader->getElements('table:table-cell', $catNode);
        }

        $colCount = $pieChart ? 2 : count($headers);
        $rowCount = count($cats);
        for ($i = 1, $index = 0; $i < $colCount; $i++, $index++) {
            $item = array();
            $item['index'] = $index;
            $item['order'] = $index;
            $item['marker'] = '';
            $item['tx'] = $xmlReader->getValue('text:p', $headers[$i]);
            $item['cat'] = array();
            $item['val'] = array();
            for ($j = 0; $j < $rowCount; $j++) {
                $item['cat'][] = $xmlReader->getValue('text:p', $cats[$j][0]);
                $item['val'][] = floatval($xmlReader->getValue('text:p', $cats[$j][$i]));
            }
            $series[] = $item;
        }

        return $series;
    }

    private function setStyle($styleName, $styleType, $style)
    {
        if (!isset($this->styles[$styleName])) {
            $this->styles[$styleName] = array();
        }
        $this->styles[$styleName][$styleType] = $style;
    }

    private function getLocalAndGlobalStyle($styleName, $styleType)
    {
        $set = function (&$dst, $key, $val) {
            if (($val !== null) && !array_key_exists($key, $dst)) {
                $dst[$key] = $val;
            }
        };
        $assign = function (&$dst, $src) use ($set) {
            foreach ($src as $k => $v) {
                $set($dst, $k, $v);
            }
        };

        if (isset($this->styles[$styleName]) && $this->styles[$styleName][$styleType]) {
            $style = $this->styles[$styleName][$styleType];
        } else {
            $style = array();
        }

        $globalStyle = Style::$styles[$styleName][$styleType];
        if (isset($globalStyle)) {
            $assign($style, $globalStyle);
        }

        return $style;
    }

    private function getColumnStyle($styleName)
    {
        if (isset($this->styles[$styleName])) {
            return $this->styles[$styleName][self::STYLE_TYPE_COLUMN];
        }
        return null;
    }

    private function getRowStyle($styleName)
    {
        if (isset($this->styles[$styleName])) {
            return $this->styles[$styleName][self::STYLE_TYPE_ROW];
        }
        return null;
    }

    private function getStyle($styleName, $styleType) {
        if (isset($this->styles[$styleName])) {
            return $this->styles[$styleName][$styleType];
        }
        return null;
    }
}
