<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element;

use PhpOffice\PhpWord\Element\Image as ImageElement;
use PhpOffice\PhpWord\Writer\HTML_ProseMirror\Style\Image as ImageStyleWriter;

/**
 * Image element HTML writer
 *
 * @since 0.10.0
 */
class Image extends Text
{
    /**
     * Write image
     *
     * @return string
     */
    public function write()
    {
        global $synoTempPath;

        if (!$this->element instanceof ImageElement) {
            return '';
        }
        /** @var \PhpOffice\PhpWord\Writer\HTML $parentWriter Type hint */
        $parentWriter = $this->parentWriter;

        if ($parentWriter->isPdf()) {
            return '';
        }

        $element = $this->element;
        $imageData = $element->getImageStringData(true);
        if ($imageData === null) {
            return '';
        }

        $elementStyle = $element->getStyle();
        $synoFileId = $elementStyle->getSynoFileId();
        $synoFileType = $elementStyle->getSynoFileType();
        if (!$synoFileType) {
            $synoFileType = 'binary';
        }
        $content = '';
        $content .= $this->writeOpening();
        $content .= $this->writeCommentRangeStart($element);
        $content .= $this->writeRevisionOpeningTag($element);
        if ($synoFileId) {
            if ('binary' === $synoFileType) {
                $content .= self::getSynoImgHtml($element);
            } else {
                $content .= self::getSynoGraphHtml($element);
            }
        } else {
            $content .= self::getSynoImgHtmlWithoutRef($element);
        }
        $content .= $this->writeRevisionClosingTag($element);
        $content .= $this->writeCommentRangeEnd($element);
        $content .= $this->writeClosing();

        if ($synoFileId) {
            // Write image file to syno temp storage
            $synoAlt = $elementStyle->getSynoAlt();
            $imgPath = "{$synoTempPath}{$synoFileId}:{$synoAlt}";
            $imgHandle = fopen($imgPath, 'wb', false);
            if ($imgHandle !== false) {
                fwrite($imgHandle, base64_decode($imageData));
                fclose($imgHandle);
            }
        }
        return $content;
    }

    /**
     * Get html of synoGraph
     *
     * @return string
     */
    static private function getSynoGraphHtml($element) {
        $elementStyle = $element->getStyle();
        $width = $elementStyle->getWidth();
        $height = $elementStyle->getHeight();
        $synoFileType = $elementStyle->getSynoFileType();
        $synoFileId = $elementStyle->getSynoFileId();
        $synoAlt = $elementStyle->getSynoAlt();
        $crop = $elementStyle->getSynoCrop();
        $config = array(
            'type'        => 'image',
            'alignment' => 'inline',
            'width'        => $width,
            'height'       => $height,
            'file_type' => $synoFileType,
            'file_id'   => $synoFileId,
            'file_alt'      => $synoAlt,
            'crop'     => $crop
        );
        return "<span synograph graph-config=\"" . htmlentities(json_encode($config)) . "\"></span>";
    }

    /**
     * Get img tag for image with reference
     *
     * @return string
     */
    static private function getSynoImgHtml($element) {
        $elementStyle = $element->getStyle();
        $styleWriter = new ImageStyleWriter($elementStyle);
        $style = $styleWriter->write();
        $synoFileType = $elementStyle->getSynoFileType();
        $synoFileId = $elementStyle->getSynoFileId();
        $synoAlt = $elementStyle->getSynoAlt();
        $content = '';
        $content .= "<img style=\"{$style}\" file-id=\"{$synoFileId}\" file-type=\"{$synoFileType}\"";
        $synoAlt = $element->getStyle()->getSynoAlt();
        if ($synoAlt) {
            $content .= " alt=\"{$synoAlt}\"";
        }
        $content .= ' />';
        return $content;
    }

    /**
     * Get img tag for image without reference
     *
     * @return string
     */
    static private function getSynoImgHtmlWithoutRef($element) {
        $elementStyle = $element->getStyle();
        $styleWriter = new ImageStyleWriter($elementStyle);
        $style = $styleWriter->write();
        $imageData = 'data:' . $element->getImageType() . ';base64,' . $imageData;
        return "<img border=\"0\" style=\"{$style}\" src=\"{$imageData}\"/>";
    }
}
